/*     */ package com.netapp.asup.data.function.sort;
/*     */ 
/*     */ import com.netapp.asup.client.arma.data.ARMAObjectInstance;
/*     */ import com.netapp.asup.data.function.FunctionAttribute;
/*     */ import com.netapp.asup.data.function.InputType;
/*     */ import com.netapp.asup.util.StringUtils;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Comparator;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.generic.GenericData.Array;
/*     */ import org.apache.avro.generic.GenericData.Record;
/*     */ import org.apache.avro.generic.GenericRecord;
/*     */ import org.apache.avro.util.Utf8;
/*     */ import org.codehaus.commons.compiler.CompileException;
/*     */ 
/*     */ public class SortInputImpl implements SortInput
/*     */ {
/*     */   private Map<String, List<SortField>> inputObject;
/*     */   
/*     */   public SortInputImpl()
/*     */   {
/*  25 */     this.inputObject = new HashMap();
/*     */   }
/*     */   
/*     */ 
/*     */   public void addAttribute(String path, SortType sortType, FunctionAttribute... sortAttrs)
/*     */   {
/*  31 */     validate(path, sortAttrs);
/*     */     
/*  33 */     List<SortField> attributeList = getAttributesList(path);
/*  34 */     attributeList.add(new SortField(sortType, sortAttrs, null));
/*     */   }
/*     */   
/*     */ 
/*     */   public void addExpression(String path, FunctionAttribute expression, SortType sortType, FunctionAttribute... sortAttrs)
/*     */   {
/*  40 */     validate(path, sortAttrs);
/*     */     
/*  42 */     if (expression == null) {
/*  43 */       throw new IllegalArgumentException("Invalid expression:" + expression);
/*     */     }
/*     */     
/*     */ 
/*  47 */     List<SortField> attributeList = getAttributesList(path);
/*  48 */     attributeList.add(new SortField(expression, sortType, sortAttrs));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <I, O> void addCustomComparator(String path, Class<? extends Comparator<ARMAObjectInstance>> comparator, Map<String, String> options)
/*     */   {
/*  57 */     validatePath(path);
/*     */     
/*  59 */     if (options == null) {
/*  60 */       options = new HashMap();
/*     */     }
/*     */     
/*  63 */     if (comparator == null) {
/*  64 */       throw new IllegalArgumentException("Invalid comparator.Null comparator:" + comparator);
/*     */     }
/*     */     
/*  67 */     List<SortField> attributeList = getAttributesList(path);
/*  68 */     attributeList.add(new SortField(comparator.getName(), options));
/*     */   }
/*     */   
/*     */   public Map<String, List<SortField>> getSortInput()
/*     */   {
/*  73 */     return this.inputObject;
/*     */   }
/*     */   
/*     */   private void validate(String path, FunctionAttribute... sortAttrs)
/*     */   {
/*  78 */     validatePath(path);
/*     */     
/*  80 */     if (sortAttrs == null) {
/*  81 */       throw new IllegalArgumentException("Invalid attributes:" + sortAttrs + ", no attributes defined");
/*     */     }
/*     */     
/*  84 */     FunctionAttribute functionAttribute = null;
/*  85 */     for (int i = 0; i < sortAttrs.length; i++) {
/*  86 */       functionAttribute = sortAttrs[i];
/*     */       
/*  88 */       if (null == functionAttribute) {
/*  89 */         throw new IllegalArgumentException("Null function attribute at index:" + i);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private void validatePath(String path)
/*     */   {
/*  97 */     if (StringUtils.isEmpty(path)) {
/*  98 */       throw new IllegalArgumentException("Path cannot be null or empty:" + path);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private List<SortField> getAttributesList(String path)
/*     */   {
/* 105 */     path = path.replaceFirst("^/", "");
/* 106 */     List<SortField> list = (List)this.inputObject.get(path);
/* 107 */     if (list == null) {
/* 108 */       list = new ArrayList();
/* 109 */       this.inputObject.put(path, list);
/*     */     }
/* 111 */     return list;
/*     */   }
/*     */   
/*     */   static class SortField
/*     */   {
/*     */     private final String expression;
/*     */     private final Class<?> expressionType;
/*     */     private final FunctionAttribute[] attributes;
/*     */     private final InputType inputType;
/*     */     private final SortType sortType;
/*     */     private final String customFunction;
/*     */     private final Map<String, String> options;
/*     */     
/*     */     private SortField(SortType sortType, FunctionAttribute... sortAttrs)
/*     */     {
/* 126 */       this.inputType = InputType.FIELD;
/* 127 */       this.sortType = sortType;
/* 128 */       this.expression = null;
/* 129 */       this.expressionType = null;
/* 130 */       this.customFunction = null;
/* 131 */       this.attributes = sortAttrs;
/* 132 */       this.options = new HashMap();
/*     */     }
/*     */     
/*     */     public SortField(FunctionAttribute expression, SortType sortType, FunctionAttribute... sortAttrs)
/*     */     {
/* 137 */       this.inputType = InputType.EXPRESSION;
/* 138 */       this.sortType = sortType;
/* 139 */       this.expression = expression.getAttrName();
/* 140 */       this.expressionType = expression.getAttrType();
/* 141 */       validateType(this.expressionType);
/* 142 */       this.attributes = sortAttrs;
/*     */       
/*     */ 
/* 145 */       compileExpression(sortAttrs);
/* 146 */       this.customFunction = null;
/* 147 */       this.options = new HashMap();
/*     */     }
/*     */     
/*     */     public SortField(String name, Map<String, String> options)
/*     */     {
/* 152 */       this.inputType = InputType.FUNCTION;
/* 153 */       this.customFunction = name;
/* 154 */       this.options = options;
/*     */       
/* 156 */       this.sortType = SortType.ASC;
/* 157 */       this.attributes = new FunctionAttribute[0];
/* 158 */       this.expression = null;
/* 159 */       this.expressionType = null;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public GenericRecord getObject(Schema sortFieldSchema)
/*     */     {
/* 168 */       GenericRecord sortField = new GenericData.Record(sortFieldSchema);
/* 169 */       sortField.put("inputType", this.inputType.getName());
/* 170 */       sortField.put("sortType", this.sortType.toString());
/*     */       
/* 172 */       Schema attrArraySchema = sortFieldSchema.getField("attributeTypes").schema();
/* 173 */       GenericData.Array<GenericRecord> attrs = new GenericData.Array(this.attributes.length, attrArraySchema);
/* 174 */       Map<Utf8, Utf8> optionsMap = new HashMap();
/*     */       
/*     */ 
/* 177 */       for (FunctionAttribute attr : this.attributes) {
/* 178 */         attrs.add(attr.getObject(attrArraySchema.getElementType()));
/*     */       }
/* 180 */       sortField.put("attributeTypes", attrs);
/*     */       
/*     */ 
/*     */ 
/* 184 */       for (String key : this.options.keySet()) {
/* 185 */         if (null != key) {
/* 186 */           String optionValue = (String)this.options.get(key);
/* 187 */           if (null != optionValue)
/* 188 */             optionsMap.put(new Utf8(key), new Utf8(optionValue));
/*     */         }
/*     */       }
/* 191 */       sortField.put("options", optionsMap);
/*     */       
/* 193 */       switch (SortInputImpl.1.$SwitchMap$com$netapp$asup$data$function$InputType[this.inputType.ordinal()]) {
/*     */       case 1: 
/* 195 */         sortField.put("expression", this.expression);
/* 196 */         sortField.put("expressionType", this.expressionType.getName());
/* 197 */         break;
/*     */       case 2: 
/* 199 */         sortField.put("customFunction", new Utf8(this.customFunction));
/*     */       }
/*     */       
/*     */       
/* 203 */       return sortField;
/*     */     }
/*     */     
/*     */     private void compileExpression(FunctionAttribute... sortAttrs)
/*     */     {
/* 208 */       int size = sortAttrs.length;
/*     */       
/* 210 */       String[] attributeNames = new String[size];
/* 211 */       Class<?>[] attributeTypes = new Class[size];
/*     */       
/* 213 */       int index = 0;
/*     */       
/* 215 */       for (FunctionAttribute attr : sortAttrs) {
/* 216 */         attributeNames[index] = attr.getAttrName();
/* 217 */         Class<?> sortAttributeType = attr.getAttrType();
/* 218 */         attributeTypes[index] = sortAttributeType;
/*     */         
/* 220 */         validateType(sortAttributeType);
/*     */         
/* 222 */         index++;
/*     */       }
/*     */       try
/*     */       {
/* 226 */         new org.codehaus.janino.ExpressionEvaluator(this.expression, Object.class, attributeNames, attributeTypes);
/*     */       }
/*     */       catch (CompileException e) {
/* 229 */         throw new IllegalArgumentException("Invalid expression " + this.expression + ", caused by :" + e.getCause(), e);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */     private void validateType(Class<?> sortAttributeType)
/*     */     {
/* 236 */       if (null == sortAttributeType) {
/* 237 */         throw new IllegalArgumentException("Null attribute type is not supported");
/*     */       }
/*     */       
/*     */ 
/* 241 */       if (!com.netapp.asup.data.function.TypeConverter.getConvertorclassmap().containsKey(sortAttributeType))
/*     */       {
/* 243 */         throw new IllegalArgumentException("Un supported attributeType.Type:" + sortAttributeType.getName());
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\function\sort\SortInputImpl.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */