/*      */ package com.netapp.asupdata.client.ARMA;
/*      */ 
/*      */ import com.netapp.asup.client.arma.ARMAObject;
/*      */ import com.netapp.asup.client.arma.data.ARMAObjectInstance;
/*      */ import com.netapp.asup.client.arma.data.ARMAObjectInstanceIterator;
/*      */ import com.netapp.asup.client.helper.ObjectDataXmlHelper;
/*      */ import com.netapp.asup.common.AsupLogger;
/*      */ import com.netapp.asup.data.function.Function;
/*      */ import com.netapp.asup.data.function.FunctionInputHandler;
/*      */ import com.netapp.asup.data.function.FunctionOutputHandler;
/*      */ import com.netapp.asup.data.function.RegisteredFunction;
/*      */ import com.netapp.asup.data.objects.FunctionData;
/*      */ import com.netapp.asup.data.objects.Identifier;
/*      */ import com.netapp.asup.data.objects.LogEventData;
/*      */ import com.netapp.asup.data.objects.ObjectData;
/*      */ import com.netapp.asup.data.objects.PerfCounters;
/*      */ import com.netapp.asup.data.pojo.ARMAPOJORegistry;
/*      */ import com.netapp.asup.data.store.BaseContainer;
/*      */ import com.netapp.asup.exception.ARMADataStoreException;
/*      */ import com.netapp.asup.exception.ARMAException;
/*      */ import com.netapp.asup.exception.ARMAFunctionExecutionException;
/*      */ import com.netapp.asup.exception.ARMAIllegalArgumentException;
/*      */ import com.netapp.asup.metamodel.MergeType;
/*      */ import com.netapp.asup.metamodel.NodeObject;
/*      */ import com.netapp.asup.metamodel.NodeObject.RelationshipType;
/*      */ import com.netapp.asup.metamodel.Relation;
/*      */ import com.netapp.asup.pojo.ARMAPojoBuilder;
/*      */ import com.netapp.asup.util.StringUtils;
/*      */ import java.io.IOException;
/*      */ import java.text.ParseException;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Arrays;
/*      */ import java.util.Collection;
/*      */ import java.util.HashMap;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Map.Entry;
/*      */ import java.util.Set;
/*      */ import java.util.TreeMap;
/*      */ import org.apache.avro.Schema;
/*      */ import org.apache.avro.Schema.Field;
/*      */ import org.apache.avro.generic.GenericData.Array;
/*      */ import org.apache.avro.generic.GenericRecord;
/*      */ import org.apache.avro.util.Utf8;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class ARMAObjectInstanceImpl
/*      */   implements ARMAObjectInstance
/*      */ {
/*      */   private final String objectName;
/*      */   private final ARMAObjectImpl currentarmaObject;
/*      */   private final Utf8 systemId;
/*      */   private final Utf8 instanceKey;
/*      */   private final Map<String, String> attributeMap;
/*      */   private final GenericRecord record;
/*   65 */   private final ARMAPOJORegistry POJORegistry = BaseContainer.getInstance().getPOJORegistry();
/*      */   
/*   67 */   private final String OPTION_VALUE = "option_value";
/*      */   
/*   69 */   private final int MAX_EVENT_SEARCH_DAYS = 90;
/*      */   
/*      */   private static final String COMMA = ",";
/*      */   
/*      */   private static final char COMPOSITE_KEY_DELIMITER = '\005';
/*      */   
/*   75 */   private static final AsupLogger LOG = AsupLogger.getLogger(ARMAObjectInstanceImpl.class);
/*      */   
/*      */ 
/*      */   public ARMAObjectInstanceImpl(ARMAObject armaObject, Utf8 systemId, Utf8 instanceKey, GenericRecord record)
/*      */   {
/*   80 */     this.currentarmaObject = ((ARMAObjectImpl)armaObject);
/*   81 */     this.objectName = this.currentarmaObject.getObjectName();
/*   82 */     this.systemId = systemId;
/*   83 */     this.instanceKey = instanceKey;
/*   84 */     this.record = record;
/*   85 */     this.attributeMap = populateAttributes();
/*      */   }
/*      */   
/*      */   public String getObjectName()
/*      */   {
/*   90 */     return this.objectName;
/*      */   }
/*      */   
/*      */   public ARMAObject getMetamodelObject() {
/*   94 */     return this.currentarmaObject;
/*      */   }
/*      */   
/*      */   public Map<String, String> getAttributes()
/*      */   {
/*   99 */     return this.attributeMap;
/*      */   }
/*      */   
/*      */   public GenericRecord getConfigRecord()
/*      */   {
/*  104 */     return (GenericRecord)this.record.get("config");
/*      */   }
/*      */   
/*      */   public GenericRecord getAttributesAsGenericRecord() {
/*  108 */     return this.record;
/*      */   }
/*      */   
/*      */   public Object getPojo(String packageName) throws ARMAException
/*      */   {
/*  113 */     ARMAPojoBuilder pojo = null;
/*      */     
/*  115 */     ARMAPojoBuilder builder = this.POJORegistry.getBuilder(packageName, this.objectName.toUpperCase());
/*  116 */     if (builder != null)
/*  117 */       pojo = builder.build(this.record, getDerivedAttributes(), String.valueOf(this.systemId), String.valueOf(this.instanceKey));
/*  118 */     if (getMetamodelObject().isOptionsAvailable()) pojo.setOptions(getOptions());
/*  119 */     return pojo;
/*      */   }
/*      */   
/*      */   private Map<String, String> populateAttributes() {
/*  123 */     Map<String, String> attrMap = new TreeMap();
/*      */     
/*  125 */     if (null != this.record)
/*      */     {
/*  127 */       populateAttributes((GenericRecord)this.record.get("config"), attrMap);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  132 */       populateAttributes((GenericRecord)this.record.get("status"), attrMap);
/*      */       
/*      */ 
/*      */ 
/*  136 */       attrMap.put("u_id", this.instanceKey.toString());
/*      */     }
/*  138 */     return attrMap;
/*      */   }
/*      */   
/*      */   private void populateAttributes(GenericRecord rec, Map<String, String> attrMap) {
/*      */     String attrName;
/*      */     String attrVal;
/*  144 */     if (null != rec) {
/*  145 */       List<Schema.Field> attrFields = rec.getSchema().getFields();
/*  146 */       attrName = null;
/*  147 */       attrVal = null;
/*  148 */       for (Schema.Field attrField : attrFields) {
/*  149 */         attrName = attrField.name();
/*  150 */         attrVal = rec.get(attrName) != null ? rec.get(attrName).toString() : null;
/*      */         
/*  152 */         attrMap.put(attrName, attrVal);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public Utf8 getSystemId() {
/*  158 */     return this.systemId;
/*      */   }
/*      */   
/*      */   public Utf8 getInstanceKey() {
/*  162 */     return this.instanceKey;
/*      */   }
/*      */   
/*      */   public String getAttribute(String attributeName) throws ARMAException
/*      */   {
/*  167 */     if (StringUtils.isEmpty(attributeName)) {
/*  168 */       throw new ARMAIllegalArgumentException(LOG, "-9999", "Attribute name is null or empty:" + attributeName);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  173 */     return (String)this.attributeMap.get(attributeName);
/*      */   }
/*      */   
/*      */   public String getDerivedAttribute(String derAttrName) throws ARMAException
/*      */   {
/*  178 */     if (isNotValidString(derAttrName)) {
/*  179 */       throw new ARMAIllegalArgumentException(LOG, "-9999", "Derived attribute name is null or empty:" + derAttrName);
/*      */     }
/*      */     
/*      */ 
/*  183 */     return this.currentarmaObject.getSession().getDerivedAttribute(this.objectName, getKey(), derAttrName);
/*      */   }
/*      */   
/*      */ 
/*      */   public String getAllAttribute(String attributeName)
/*      */     throws ARMAException
/*      */   {
/*  190 */     if (StringUtils.isEmpty(attributeName)) {
/*  191 */       throw new ARMAIllegalArgumentException(LOG, "-9999", "Attribute name is null or empty:" + attributeName);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  196 */     String attributeValue = null;
/*  197 */     if (this.attributeMap.keySet().contains(attributeName)) {
/*  198 */       attributeValue = getAttribute(attributeName);
/*      */     } else {
/*  200 */       attributeValue = getDerivedAttribute(attributeName);
/*      */     }
/*  202 */     return attributeValue;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private Utf8 getGenericRecordAttributeValue(GenericRecord record, String parentKeyAttrName)
/*      */   {
/*  209 */     Schema.Field field = record.getSchema().getField("config");
/*      */     
/*      */ 
/*  212 */     if (null != field) {
/*  213 */       String fieldName = field.name();
/*      */       
/*  215 */       GenericRecord configRecord = (GenericRecord)record.get(fieldName);
/*  216 */       Schema configRecordSchema = configRecord.getSchema();
/*  217 */       Schema.Field parentField = configRecordSchema.getField(parentKeyAttrName);
/*  218 */       if (parentField != null) {
/*  219 */         return (Utf8)configRecord.get(parentField.name());
/*      */       }
/*      */     }
/*      */     
/*  223 */     return null;
/*      */   }
/*      */   
/*      */   public ARMAObjectInstance getParentInstance(String objectName)
/*      */     throws ARMAException
/*      */   {
/*  229 */     if (StringUtils.isEmpty(objectName)) {
/*  230 */       throw new ARMAIllegalArgumentException(LOG, "-9999", "Invalid Parent name.Null or empty:" + objectName);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  235 */     NodeObject currNodeObject = this.currentarmaObject.getNodeObject();
/*  236 */     String realName = objectName;
/*  237 */     String keyName = null;
/*  238 */     if (currNodeObject.getVirtualVsRealMap().containsKey(objectName)) realName = (String)currNodeObject.getVirtualVsRealMap().get(objectName);
/*  239 */     ARMAObjectImpl parent = (ARMAObjectImpl)this.currentarmaObject.getParentFromStore(realName);
/*      */     
/*      */ 
/*  242 */     if (parent == null)
/*      */     {
/*  244 */       parent = (ARMAObjectImpl)this.currentarmaObject.getParent(realName);
/*      */       
/*  246 */       if (parent == null) {
/*  247 */         throw new ARMAException(LOG, "-9999", " Parent does not exist by the name: " + realName);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  253 */     Relation relation = currNodeObject.getRelationMap(realName);
/*  254 */     if (relation.getType() == NodeObject.RelationshipType.MULTI_RELATION)
/*      */     {
/*  256 */       List<Relation> childRelations = relation.getRelations();
/*  257 */       for (Relation childRelation : childRelations) {
/*  258 */         if (childRelation.getVirtualNodeName().equals(objectName)) { keyName = childRelation.getKeyName(); break;
/*      */         }
/*      */       }
/*      */     }
/*  262 */     else if (relation.getKeyName() != null) { keyName = relation.getKeyName();
/*  263 */     } else { keyName = parent.getKeyAttribute();
/*      */     }
/*      */     
/*      */ 
/*  267 */     Utf8 parentKeyAttributeValue = null;
/*  268 */     String[] multipleParentIds = keyName.split(",");
/*      */     
/*  270 */     if (multipleParentIds.length > 1)
/*      */     {
/*  272 */       List<String> multipleIds = Arrays.asList(multipleParentIds);
/*  273 */       Iterator<String> listIt = multipleIds.iterator();
/*  274 */       Utf8 parentIdVal = null;
/*  275 */       StringBuilder builder = new StringBuilder();
/*      */       
/*  277 */       while (listIt.hasNext()) {
/*  278 */         parentIdVal = getGenericRecordAttributeValue(this.record, (String)listIt.next());
/*  279 */         if (parentIdVal != null) {
/*  280 */           builder.append(parentIdVal.toString());
/*      */         }
/*  282 */         if (listIt.hasNext()) {
/*  283 */           builder.append('\005');
/*      */         }
/*      */       }
/*  286 */       parentKeyAttributeValue = new Utf8(builder.toString());
/*      */     } else {
/*  288 */       String parentKeyAttrName = multipleParentIds[0];
/*  289 */       parentKeyAttributeValue = getGenericRecordAttributeValue(this.record, parentKeyAttrName);
/*      */       
/*  291 */       if ((parentKeyAttributeValue == null) && 
/*  292 */         ("SYSTEM".equalsIgnoreCase(objectName)) && ("sys_id".equalsIgnoreCase(parentKeyAttrName))) {
/*  293 */         parentKeyAttributeValue = this.systemId;
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  299 */     if (parentKeyAttributeValue == null)
/*      */     {
/*      */ 
/*  302 */       return null;
/*      */     }
/*      */     
/*  305 */     Map<Utf8, Map<Utf8, GenericRecord>> systemToGenericRecord = parent.getObjectData().getSystemToGenericRecord();
/*  306 */     Map<Utf8, GenericRecord> instanceMap = (Map)systemToGenericRecord.get(this.systemId);
/*      */     
/*  308 */     if (LOG.isDebugEnabled()) {
/*  309 */       LOG.debug("-9999", "parent key attribute value from current instance : " + parentKeyAttributeValue);
/*      */       
/*      */ 
/*      */ 
/*  313 */       LOG.debug("-9999", "parent instance map : " + instanceMap);
/*      */     }
/*      */     
/*  316 */     GenericRecord parentGenericRecord = (GenericRecord)instanceMap.get(parentKeyAttributeValue);
/*  317 */     if (parentGenericRecord != null) {
/*  318 */       ARMAObjectInstance parentInstance = new ARMAObjectInstanceImpl(parent, this.systemId, parentKeyAttributeValue, parentGenericRecord);
/*      */       
/*      */ 
/*  321 */       return parentInstance;
/*      */     }
/*  323 */     throw new ARMAException(LOG, "-9999", "Parent instance not found for objectName :" + objectName);
/*      */   }
/*      */   
/*      */ 
/*      */   public ARMAObjectInstanceIterator getChildInstances(String objectName)
/*      */     throws ARMAException
/*      */   {
/*  330 */     if (StringUtils.isEmpty(objectName)) {
/*  331 */       throw new ARMAIllegalArgumentException(LOG, "-9999", "Null or empty objectName parameter is not supported in the getChildInstances call ");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  337 */     if (this.attributeMap.size() == 0) {
/*  338 */       throw new ARMAException(LOG, "-9999", " can't invoke get parent or child instance on an empty object ");
/*      */     }
/*      */     
/*      */ 
/*  342 */     Map<Utf8, Map<Utf8, GenericRecord>> childSystemToGenericRecord = new HashMap();
/*      */     
/*  344 */     Map<Utf8, GenericRecord> childInstances = new HashMap();
/*      */     
/*  346 */     boolean isVirtual = false;
/*  347 */     Utf8 parentId = null;
/*  348 */     ARMAObjectImpl metaObject = this.currentarmaObject;
/*  349 */     String realObjectName = null;
/*  350 */     NodeObject nodeObject = metaObject.getNodeObject();
/*  351 */     if (nodeObject.getVirtualVsRealMap().containsKey(objectName)) { isVirtual = true;realObjectName = (String)nodeObject.getVirtualVsRealMap().get(objectName);
/*  352 */     } else { realObjectName = objectName;
/*      */     }
/*      */     
/*      */ 
/*  356 */     ARMAObjectImpl childObject = (ARMAObjectImpl)metaObject.getChildFromStore(realObjectName);
/*      */     
/*      */ 
/*  359 */     if (childObject == null)
/*      */     {
/*  361 */       childObject = (ARMAObjectImpl)metaObject.getChild(realObjectName);
/*  362 */       if (childObject == null)
/*      */       {
/*  364 */         throw new ARMAException(LOG, "-9999", "A child with name " + objectName + " does not exist for current object, or the child object has not be navigated to by a path in the metamodel graph");
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  373 */     ObjectData childObjectData = childObject.getObjectData();
/*  374 */     if (childObjectData == null) {
/*  375 */       childObjectData = metaObject.getSession().getObject(realObjectName, nodeObject);
/*      */     }
/*      */     
/*      */ 
/*  379 */     Map<Utf8, List<Utf8>> childKeyIndex = childObjectData.getKeyIndex();
/*      */     
/*      */ 
/*  382 */     Map<Utf8, List<Utf8>> currentKeyIndex = metaObject.getObjectData().getKeyIndex();
/*      */     
/*      */ 
/*  385 */     boolean populateIndex = false;
/*  386 */     if ((childObjectData.getIndexMap().size() == 0) || (childObjectData.getIndexMap().get(new Utf8(this.objectName)) == null)) { populateIndex = true;
/*      */     }
/*  388 */     if (populateIndex) { childObjectData.populateIndexesForParent(nodeObject);
/*      */     }
/*  390 */     Map<Utf8, Map<Utf8, Map<Utf8, GenericData.Array<Utf8>>>> indexMap = childObjectData.getIndexMap();
/*      */     
/*      */ 
/*  393 */     Map<Utf8, Map<Utf8, GenericData.Array<Utf8>>> parentIdMap = (Map)indexMap.get(new Utf8(getObjectName()));
/*      */     
/*  395 */     if (isVirtual) parentId = new Utf8(objectName + "_" + this.instanceKey.toString()); else { parentId = this.instanceKey;
/*      */     }
/*  397 */     if ((parentIdMap == null) || (null == parentIdMap.get(parentId)) || (((Map)parentIdMap.get(parentId)).isEmpty())) {
/*  398 */       childSystemToGenericRecord.put(this.systemId, childInstances);
/*  399 */       LOG.warn("-9999", "No:" + objectName + " child instances for object with name : " + getObjectName() + " returning empty iterator");
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  405 */       return new ARMAObjectInstanceIteratorImpl(childObject, childSystemToGenericRecord, new HashMap());
/*      */     }
/*      */     
/*  408 */     Map<Utf8, GenericData.Array<Utf8>> childMap = (Map)parentIdMap.get(parentId);
/*      */     
/*      */ 
/*  411 */     List<Utf8> childInstanceKeys = new ArrayList();
/*      */     
/*  413 */     if (!currentKeyIndex.isEmpty())
/*      */     {
/*  415 */       if (childKeyIndex.isEmpty()) {
/*  416 */         return new ARMAObjectInstanceIteratorImpl(childObject, getChildGenricRecord(childObjectData, currentKeyIndex, childMap), childKeyIndex);
/*      */       }
/*      */       
/*      */ 
/*  420 */       return new ARMAObjectInstanceIteratorImpl(childObject, getChildGenricRecord(childObjectData, currentKeyIndex, childMap), getChildKeyIndex(currentKeyIndex, childMap));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  425 */     childInstanceKeys = (List)childMap.get(this.systemId);
/*      */     
/*  427 */     if ((childInstanceKeys == null) || (childInstanceKeys.size() == 0)) {
/*  428 */       childSystemToGenericRecord.put(this.systemId, childInstances);
/*  429 */       LOG.warn("-9999", "No child instance keys for this object instance, returning an empty child instance list");
/*      */       
/*      */ 
/*  432 */       return new ARMAObjectInstanceIteratorImpl(childObject, childSystemToGenericRecord, new HashMap());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  438 */     Map<Utf8, GenericRecord> childObjectInstances = (Map)childObjectData.getSystemToGenericRecord().get(this.systemId);
/*      */     
/*      */ 
/*  441 */     if (childObjectInstances == null) {
/*  442 */       childSystemToGenericRecord.put(this.systemId, childInstances);
/*  443 */       LOG.warn("-9999", "No child object data for " + getObjectName() + " with system id :" + this.systemId + " returning an empty child instance list ");
/*      */       
/*      */ 
/*      */ 
/*  447 */       return new ARMAObjectInstanceIteratorImpl(childObject, childSystemToGenericRecord, childKeyIndex);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  452 */     for (Utf8 childInstanceKey : childInstanceKeys) {
/*  453 */       childInstances.put(childInstanceKey, childObjectInstances.get(childInstanceKey));
/*      */     }
/*      */     
/*      */ 
/*  457 */     childSystemToGenericRecord.put(this.systemId, childInstances);
/*      */     
/*  459 */     return new ARMAObjectInstanceIteratorImpl(childObject, childSystemToGenericRecord, childKeyIndex);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private Map<Utf8, List<Utf8>> getChildKeyIndex(Map<Utf8, List<Utf8>> currentKeyIndex, Map<Utf8, GenericData.Array<Utf8>> childMap)
/*      */   {
/*  467 */     Map<Utf8, List<Utf8>> childKeyIndex = new HashMap();
/*  468 */     List<Utf8> sysIds = (List)currentKeyIndex.get(this.instanceKey);
/*      */     
/*  470 */     for (Iterator i$ = sysIds.iterator(); i$.hasNext();) { sysId = (Utf8)i$.next();
/*  471 */       GenericData.Array<Utf8> instances = (GenericData.Array)childMap.get(sysId);
/*  472 */       if (null != instances)
/*      */       {
/*      */ 
/*  475 */         for (Utf8 instaceId : instances)
/*  476 */           populate(childKeyIndex, instaceId, sysId); }
/*      */     }
/*      */     Utf8 sysId;
/*  479 */     return childKeyIndex;
/*      */   }
/*      */   
/*      */ 
/*      */   private void populate(Map<Utf8, List<Utf8>> childKeyIndex, Utf8 instaceId, Utf8 sysId)
/*      */   {
/*  485 */     if (childKeyIndex.containsKey(instaceId)) {
/*  486 */       List<Utf8> childSysIds = (List)childKeyIndex.get(instaceId);
/*  487 */       if (!childSysIds.contains(sysId)) {
/*  488 */         childSysIds.add(sysId);
/*      */       }
/*      */     } else {
/*  491 */       List<Utf8> childSys = new ArrayList();
/*  492 */       childSys.add(sysId);
/*  493 */       childKeyIndex.put(instaceId, childSys);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private Map<Utf8, Map<Utf8, GenericRecord>> getChildGenricRecord(ObjectData childObjectData, Map<Utf8, List<Utf8>> currentKeyIndex, Map<Utf8, GenericData.Array<Utf8>> childMap)
/*      */   {
/*  500 */     Map<Utf8, Map<Utf8, GenericRecord>> childSystemToGenericRecord = new HashMap();
/*  501 */     List<Utf8> sysIds = (List)currentKeyIndex.get(this.instanceKey);
/*      */     
/*      */ 
/*  504 */     for (Utf8 sysId : sysIds) {
/*  505 */       Map<Utf8, GenericRecord> childObjectInstances = new HashMap();
/*  506 */       GenericData.Array<Utf8> instances = (GenericData.Array)childMap.get(sysId);
/*  507 */       if (null != instances)
/*      */       {
/*      */ 
/*  510 */         for (Utf8 instaceId : instances) {
/*  511 */           childObjectInstances.put(instaceId, ((Map)childObjectData.getSystemToGenericRecord().get(sysId)).get(instaceId));
/*      */         }
/*      */         
/*  514 */         childSystemToGenericRecord.put(sysId, childObjectInstances);
/*      */       } }
/*  516 */     return childSystemToGenericRecord;
/*      */   }
/*      */   
/*      */   public Map<String, String> getOptions()
/*      */     throws ARMAException
/*      */   {
/*  522 */     Map<String, String> optionAttributes = new HashMap();
/*  523 */     ARMAObjectImpl metaObject = this.currentarmaObject;
/*  524 */     ARMAClientSession session = metaObject.getSession();
/*  525 */     String requestId = session.getRequestId();
/*      */     
/*  527 */     if (!metaObject.isOptionsAvailable()) {
/*  528 */       LOG.warn(requestId, "Options are supported for this object, returning empty map");
/*      */       
/*  530 */       return optionAttributes;
/*      */     }
/*      */     
/*  533 */     Map<Utf8, GenericRecord> optionMap = null;
/*  534 */     Utf8 optionGroupId = session.getObject(this.objectName).getOptionGroupIdVal(this.record);
/*      */     
/*      */ 
/*  537 */     if (optionGroupId == null) {
/*  538 */       LOG.warn(requestId, "Option group id is null for current object, returning empty option map");
/*      */       
/*  540 */       return optionAttributes;
/*      */     }
/*      */     try
/*      */     {
/*  544 */       optionMap = session.getOptions(this.objectName, optionGroupId);
/*      */     }
/*      */     catch (ARMADataStoreException e) {
/*  547 */       throw new ARMAException(LOG, requestId, "Error occured while trying to fetch options data:" + this.objectName, e);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  552 */     if ((optionMap != null) && (optionMap.size() > 0)) {
/*  553 */       Object optionValue = null;
/*  554 */       for (Map.Entry<Utf8, GenericRecord> mapEntry : optionMap.entrySet()) {
/*  555 */         optionValue = ((GenericRecord)mapEntry.getValue()).get("option_value");
/*  556 */         optionAttributes.put(((Utf8)mapEntry.getKey()).toString(), optionValue != null ? optionValue.toString() : null);
/*      */       }
/*      */       
/*      */ 
/*  560 */       return optionAttributes;
/*      */     }
/*  562 */     LOG.info(requestId, "Options is null or empty. Returning empty options map.Option group id:" + optionGroupId.toString());
/*      */     
/*      */ 
/*  565 */     return optionAttributes;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public GenericRecord executeGenericIOFunction(RegisteredFunction<GenericRecord, GenericRecord> registeredFunction, GenericRecord input)
/*      */     throws ARMAException
/*      */   {
/*  573 */     Function function = registeredFunction.getFunction();
/*      */     try {
/*  575 */       return function.execute(this, input, registeredFunction.getOutputSchema());
/*      */     }
/*      */     catch (Throwable e) {
/*  578 */       RuntimeException exception = new RuntimeException(e);
/*  579 */       throw new ARMAFunctionExecutionException(LOG, registeredFunction.toString(), "Function execution failed,Reason:" + e.getMessage(), exception);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <I, O> O executeFunction(RegisteredFunction<I, O> registeredFunction, I input)
/*      */     throws ARMAException
/*      */   {
/*  591 */     FunctionInputHandler<I> inputHandler = registeredFunction.getInputHandler();
/*      */     
/*  593 */     GenericRecord functionInput = null;
/*  594 */     if (input != null) {
/*  595 */       if (inputHandler != null) {
/*  596 */         functionInput = inputHandler.handleInput(registeredFunction.getInputSchema(), input);
/*      */       }
/*  598 */       else if ((functionInput instanceof GenericRecord)) {
/*  599 */         functionInput = (GenericRecord)input;
/*      */       } else {
/*  601 */         throw new ARMAFunctionExecutionException(LOG, this.currentarmaObject.getSession().getRequestId(), "Neither a valid input handler is defined, nor the function input is a instance of GenericRecord");
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  607 */     GenericRecord output = executeGenericIOFunction(registeredFunction, functionInput);
/*      */     
/*      */ 
/*      */ 
/*  611 */     FunctionOutputHandler<O> outputHandler = registeredFunction.getOutputHandler();
/*      */     
/*  613 */     if (outputHandler != null) {
/*  614 */       return (O)outputHandler.handleOutput(output);
/*      */     }
/*  616 */     return output;
/*      */   }
/*      */   
/*      */ 
/*      */   public Utf8 getKey()
/*      */   {
/*  622 */     return new Utf8(String.valueOf(this.systemId) + "_" + String.valueOf(this.instanceKey));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public GenericRecord getGenericIOFunctionOutput(String identifier, RegisteredFunction<GenericRecord, GenericRecord> registeredFunction)
/*      */     throws ARMAException
/*      */   {
/*  632 */     if (isNotValidString(identifier)) {
/*  633 */       throw new ARMAIllegalArgumentException(LOG, registeredFunction.toString(), "Function identifier is null or empty:" + identifier);
/*      */     }
/*      */     
/*      */ 
/*  637 */     FunctionData functionData = this.currentarmaObject.getSession().getFunctionData(this.objectName, identifier, registeredFunction);
/*      */     
/*      */ 
/*      */     try
/*      */     {
/*  642 */       return functionData.getFunctionOutputForInstance(getKey());
/*      */     } catch (IOException e) {
/*  644 */       throw new ARMAException(LOG, registeredFunction.toString(), "Error in getting function data caused by :" + e.getMessage(), e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <I, O> O getFunctionOutput(String identifier, RegisteredFunction<I, O> registeredFunction)
/*      */     throws ARMAException
/*      */   {
/*  655 */     if (isNotValidString(identifier)) {
/*  656 */       throw new ARMAIllegalArgumentException(LOG, registeredFunction.toString(), "Function identifier is null or empty:" + identifier);
/*      */     }
/*      */     
/*      */ 
/*  660 */     FunctionData functionData = this.currentarmaObject.getSession().getFunctionData(this.objectName, identifier, registeredFunction);
/*      */     
/*      */     try
/*      */     {
/*  664 */       GenericRecord functionOutputForInstance = functionData.getFunctionOutputForInstance(getKey());
/*      */       
/*  666 */       FunctionOutputHandler<O> outputHandler = registeredFunction.getOutputHandler();
/*      */       
/*  668 */       if (outputHandler != null) {
/*  669 */         return (O)outputHandler.handleOutput(functionOutputForInstance);
/*      */       }
/*      */       
/*  672 */       return functionOutputForInstance;
/*      */     }
/*      */     catch (IOException e) {
/*  675 */       throw new ARMAException(LOG, registeredFunction.toString(), "Error in getting function data caused by :" + e.getMessage(), e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public void addDerviedAttribute(String derAttrName, String derAttrValue)
/*      */     throws ARMAException
/*      */   {
/*  685 */     if (isNotValidString(derAttrName)) {
/*  686 */       throw new ARMAIllegalArgumentException(LOG, "-9999", "Derived attribute name is null or empty:" + derAttrName);
/*      */     }
/*      */     
/*      */ 
/*  690 */     if (isNotValidString(derAttrValue)) {
/*  691 */       throw new ARMAIllegalArgumentException(LOG, "-9999", "Derived attribute value is null or empty:" + derAttrValue);
/*      */     }
/*      */     
/*      */ 
/*  695 */     this.currentarmaObject.getSession().addDerivedAttribute(this.objectName, getKey(), derAttrName, derAttrValue);
/*      */   }
/*      */   
/*      */   public Map<String, String> getDerivedAttributes()
/*      */     throws ARMAException
/*      */   {
/*  701 */     Map<Utf8, Utf8> allDerivedAttributes = this.currentarmaObject.getSession().getAllDerivedAttribute(this.objectName, getKey());
/*      */     
/*      */ 
/*  704 */     Map<String, String> derivedAttributesMap = new HashMap();
/*      */     
/*  706 */     for (Utf8 derivedAttr : allDerivedAttributes.keySet()) {
/*  707 */       derivedAttributesMap.put(derivedAttr.toString(), ((Utf8)allDerivedAttributes.get(derivedAttr)).toString());
/*      */     }
/*      */     
/*      */ 
/*  711 */     return derivedAttributesMap;
/*      */   }
/*      */   
/*      */   public Map<String, String> getAllAttributes() throws ARMAException
/*      */   {
/*  716 */     Map<String, String> attributes = new HashMap();
/*  717 */     attributes.putAll(getAttributes());
/*  718 */     attributes.putAll(getDerivedAttributes());
/*  719 */     return attributes;
/*      */   }
/*      */   
/*      */   private boolean isNotValidString(String identifier) {
/*  723 */     return StringUtils.isEmpty(identifier);
/*      */   }
/*      */   
/*      */   public boolean isParent(String parentObjectName)
/*      */   {
/*  728 */     if (isNotValidString(parentObjectName)) {
/*  729 */       return false;
/*      */     }
/*  731 */     NodeObject parentByName = this.currentarmaObject.getNodeObject().getParentByName(parentObjectName.toLowerCase());
/*      */     
/*  733 */     if (parentByName != null) {
/*  734 */       return true;
/*      */     }
/*  736 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */   public boolean isChild(String childObjectName)
/*      */   {
/*  742 */     if (isNotValidString(childObjectName))
/*  743 */       return false;
/*  744 */     NodeObject childByName = this.currentarmaObject.getNodeObject().getChildByName(childObjectName.toLowerCase());
/*      */     
/*  746 */     if (childByName != null) {
/*  747 */       return true;
/*      */     }
/*  749 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */   public String getSessionId()
/*      */   {
/*  755 */     return this.currentarmaObject.getRequestId();
/*      */   }
/*      */   
/*      */   public <I, O> RegisteredFunction<I, O> getRegisteredFunction(String functionName)
/*      */     throws ARMAException
/*      */   {
/*  761 */     return this.currentarmaObject.getRegisteredFunction(functionName);
/*      */   }
/*      */   
/*      */   public RegisteredFunction<GenericRecord, GenericRecord> getGenericIORegisteredFunction(String functionName)
/*      */     throws ARMAException
/*      */   {
/*  767 */     return this.currentarmaObject.getGenericIORegisteredFunction(functionName);
/*      */   }
/*      */   
/*      */   public List<Long> getPerfTimeStamps(String counterObjectName, int intervalInDays)
/*      */     throws ARMAException
/*      */   {
/*  773 */     ARMAClientSession session = this.currentarmaObject.getSession();
/*  774 */     PerfCounters perfData = session.getPerfData(this.objectName, counterObjectName, intervalInDays);
/*      */     
/*      */     try
/*      */     {
/*  778 */       return perfData.getTimeStamp(session.getIdentifier(), this.systemId, counterObjectName, Integer.valueOf(intervalInDays));
/*      */     }
/*      */     catch (ParseException e) {
/*  781 */       throw new ARMAException(LOG, null, "unable to parse timestamp : ", e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   public List<Long> getPerfTimeStamps(String timeStamp, String counterObjectName, int intervalInDays)
/*      */     throws ARMAException
/*      */   {
/*  789 */     this.currentarmaObject.getSession().getIdentifier().setTimeStamp(timeStamp);
/*  790 */     return getPerfTimeStamps(counterObjectName, intervalInDays);
/*      */   }
/*      */   
/*      */   public Map<String, List<String>> getRawPerfData(String counterObjectName, String counterName, int intervalInDays) throws ARMAException
/*      */   {
/*  795 */     ARMAClientSession session = this.currentarmaObject.getSession();
/*  796 */     PerfCounters perfData = session.getPerfData(counterObjectName, this.objectName, intervalInDays);
/*      */     try {
/*  798 */       return perfData.getCounterData(session.getIdentifier(), this.systemId, counterObjectName, counterName, Integer.valueOf(intervalInDays));
/*      */     }
/*      */     catch (ParseException e) {
/*  801 */       throw new ARMAException(LOG, null, "unable to parse timestamp : ", e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public GenericRecord getRawLogData(int intervalInDays, String eventType, String crc)
/*      */     throws ARMAException
/*      */   {
/*  810 */     ARMAClientSession session = this.currentarmaObject.getSession();
/*  811 */     LogEventData logData = session.getLogData(this.objectName, intervalInDays, new String[] { eventType });
/*  812 */     return logData.getRawEventsForCrc(this.systemId, new Utf8(this.objectName), this.instanceKey, new Utf8(eventType), new Utf8(crc));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public Map<String, String> getLogAttributes(int intervalInDays, String eventType, String crc)
/*      */     throws ARMAException
/*      */   {
/*  820 */     GenericRecord logRecord = getRawLogData(intervalInDays, eventType, crc);
/*      */     Map<String, String> attrMap;
/*  822 */     Map<String, String> attrMap; if (logRecord != null)
/*      */     {
/*  824 */       attrMap = LogEventData.getLogAttributes(logRecord);
/*      */     }
/*      */     else {
/*  827 */       attrMap = new TreeMap();
/*      */     }
/*  829 */     return attrMap;
/*      */   }
/*      */   
/*      */   public Collection<GenericRecord> getRawLogDataForAllCRCs(int intervalInDays, String... eventTypes) throws ARMAException {
/*  833 */     ARMAClientSession session = this.currentarmaObject.getSession();
/*  834 */     LogEventData logData = session.getLogData(this.objectName, intervalInDays, eventTypes);
/*  835 */     return logData.getRawEventsForTypes(this.systemId, new Utf8(this.objectName), this.instanceKey, eventTypes);
/*      */   }
/*      */   
/*      */ 
/*      */   public Map<String, Map<String, Map<String, Map<String, Map<String, String>>>>> getAllLogDataForSystem(int intervalInDays)
/*      */     throws ARMAException
/*      */   {
/*  842 */     LogEventData logData = getLogEventData(intervalInDays);
/*  843 */     Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> allEventsForSystem = logData.getAllEventsForSystem(this.systemId);
/*  844 */     Map<String, Map<String, Map<String, Map<String, Map<String, String>>>>> objectMap = new HashMap();
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  849 */     String objectName = null;
/*  850 */     String instanceKey = null;
/*  851 */     String eventType = null;
/*  852 */     String crcKey = null;
/*      */     
/*  854 */     if ((allEventsForSystem == null) || (allEventsForSystem.size() == 0)) { return objectMap;
/*      */     }
/*  856 */     for (Map.Entry<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> objectEntrySet : allEventsForSystem.entrySet()) {
/*  857 */       objectName = ((Utf8)objectEntrySet.getKey()).toString();
/*  858 */       Map<String, Map<String, Map<String, Map<String, String>>>> instanceMap = (Map)objectMap.get(objectName);
/*  859 */       if (instanceMap == null)
/*  860 */         instanceMap = new HashMap();
/*  861 */       for (Map.Entry<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>> instanceEntrySet : ((Map)objectEntrySet.getValue()).entrySet()) {
/*  862 */         instanceKey = ((Utf8)instanceEntrySet.getKey()).toString();
/*  863 */         Map<String, Map<String, Map<String, String>>> eventMap = (Map)instanceMap.get(instanceKey);
/*  864 */         if (eventMap == null)
/*  865 */           eventMap = new HashMap();
/*  866 */         for (Map.Entry<Utf8, Map<Utf8, GenericRecord>> eventEntrySet : ((Map)instanceEntrySet.getValue()).entrySet()) {
/*  867 */           eventType = ((Utf8)eventEntrySet.getKey()).toString();
/*  868 */           Map<String, Map<String, String>> crcMap = (Map)eventMap.get(eventType);
/*  869 */           if (crcMap == null)
/*  870 */             crcMap = new HashMap();
/*  871 */           for (Map.Entry<Utf8, GenericRecord> crcEntrySet : ((Map)eventEntrySet.getValue()).entrySet()) {
/*  872 */             crcKey = ((Utf8)crcEntrySet.getKey()).toString();
/*  873 */             Map<String, String> logAttributesMap = LogEventData.getLogAttributes((GenericRecord)crcEntrySet.getValue());
/*  874 */             crcMap.put(crcKey, logAttributesMap);
/*      */           }
/*  876 */           eventMap.put(eventType, crcMap);
/*      */         }
/*      */         
/*  879 */         instanceMap.put(instanceKey, eventMap);
/*      */       }
/*  881 */       objectMap.put(objectName, instanceMap);
/*      */     }
/*  883 */     return objectMap;
/*      */   }
/*      */   
/*      */   private LogEventData getLogEventData(int intervalInDays) throws ARMAException {
/*  887 */     if ((intervalInDays > 90) || (intervalInDays <= 0))
/*      */     {
/*  889 */       LOG.warn(null, String.format("Invalid range for event search: %d, setting to: %d", new Object[] { Integer.valueOf(intervalInDays), Integer.valueOf(90) }));
/*  890 */       intervalInDays = 90;
/*      */     }
/*      */     
/*  893 */     ARMAClientSession session = this.currentarmaObject.getSession();
/*  894 */     LogEventData logData = session.getLogData(this.objectName, intervalInDays, new String[0]);
/*  895 */     return logData;
/*      */   }
/*      */   
/*      */ 
/*      */   public Map<String, Map<String, Map<String, String>>> getAllLogData(int intervalInDays)
/*      */     throws ARMAException
/*      */   {
/*  902 */     Map<String, Map<String, Map<String, String>>> logAttributeMap = new HashMap();
/*  903 */     LogEventData logData = getLogEventData(intervalInDays);
/*  904 */     Map<Utf8, Map<Utf8, GenericRecord>> eventMap = logData.getEventsForObjectInstance(this.systemId, new Utf8(this.objectName), this.instanceKey);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  910 */     if (eventMap != null)
/*      */     {
/*  912 */       for (Utf8 eventType : eventMap.keySet())
/*      */       {
/*  914 */         Map<Utf8, GenericRecord> crcMap = (Map)eventMap.get(eventType);
/*  915 */         Map<String, Map<String, String>> attributeEventTypeMap = new HashMap();
/*  916 */         for (Map.Entry<Utf8, GenericRecord> e : crcMap.entrySet())
/*      */         {
/*  918 */           Utf8 crc = (Utf8)e.getKey();
/*  919 */           GenericRecord logRecord = (GenericRecord)e.getValue();
/*  920 */           attributeEventTypeMap.put(crc.toString(), LogEventData.getLogAttributes(logRecord));
/*      */         }
/*  922 */         logAttributeMap.put(eventType.toString(), attributeEventTypeMap);
/*      */       }
/*      */     }
/*      */     
/*  926 */     return logAttributeMap;
/*      */   }
/*      */   
/*      */   public List<Map<String, String>> getLogAttributesForAllEvents(int intervalInDays) throws ARMAException
/*      */   {
/*  931 */     if ((intervalInDays > 90) || (intervalInDays <= 0))
/*      */     {
/*  933 */       LOG.warn(null, String.format("Invalid range for event search: %d, setting to: %d", new Object[] { Integer.valueOf(intervalInDays), Integer.valueOf(90) }));
/*  934 */       intervalInDays = 90;
/*      */     }
/*  936 */     List<Map<String, String>> logAttributeList = new ArrayList();
/*  937 */     ARMAClientSession session = this.currentarmaObject.getSession();
/*  938 */     LogEventData logData = session.getLogData(this.objectName, intervalInDays, new String[0]);
/*  939 */     Map<Utf8, Map<Utf8, GenericRecord>> eventMap = logData.getEventsForObjectInstance(this.systemId, new Utf8(this.objectName), this.instanceKey);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  945 */     if (eventMap != null)
/*      */     {
/*  947 */       for (Utf8 eventType : eventMap.keySet())
/*      */       {
/*  949 */         Map<Utf8, GenericRecord> crcMap = (Map)eventMap.get(eventType);
/*      */         
/*  951 */         for (Map.Entry<Utf8, GenericRecord> e : crcMap.entrySet())
/*      */         {
/*  953 */           Utf8 crc = (Utf8)e.getKey();
/*  954 */           GenericRecord logRecord = (GenericRecord)e.getValue();
/*  955 */           logAttributeList.add(LogEventData.getLogAttributes(logRecord));
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  961 */     return logAttributeList;
/*      */   }
/*      */   
/*      */   public List<Map<String, String>> getAllLogDataForSeverity(int intervalInDays, String severity) throws ARMAException
/*      */   {
/*  966 */     if ((intervalInDays > 90) || (intervalInDays <= 0))
/*      */     {
/*  968 */       LOG.warn(null, String.format("Invalid range for event search: %d, setting to: %d", new Object[] { Integer.valueOf(intervalInDays), Integer.valueOf(90) }));
/*  969 */       intervalInDays = 90;
/*      */     }
/*  971 */     List<Map<String, String>> logAttributeList = new ArrayList();
/*  972 */     ARMAClientSession session = this.currentarmaObject.getSession();
/*  973 */     LogEventData logData = session.getLogData(this.objectName, intervalInDays, new String[0]);
/*  974 */     Map<Utf8, Map<Utf8, GenericRecord>> eventMap = logData.getEventsForObjectInstance(this.systemId, new Utf8(this.objectName), this.instanceKey);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  980 */     if (eventMap != null)
/*      */     {
/*  982 */       for (Utf8 eventType : eventMap.keySet())
/*      */       {
/*  984 */         Map<Utf8, GenericRecord> crcMap = (Map)eventMap.get(eventType);
/*      */         
/*  986 */         for (Map.Entry<Utf8, GenericRecord> e : crcMap.entrySet())
/*      */         {
/*  988 */           Utf8 crc = (Utf8)e.getKey();
/*  989 */           GenericRecord logRecord = (GenericRecord)e.getValue();
/*  990 */           Map<String, String> valueMap = LogEventData.getLogAttributes(logRecord);
/*  991 */           if ((valueMap != null) && (((String)valueMap.get("severity")).equalsIgnoreCase(severity)))
/*      */           {
/*  993 */             logAttributeList.add(valueMap);
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1001 */     return logAttributeList;
/*      */   }
/*      */   
/*      */   public List<Map<String, String>> getLogAttributesForAllCRCs(int intervalInDays, String... eventTypes) throws ARMAException
/*      */   {
/* 1006 */     Collection<GenericRecord> rawLogsForAllCRCs = getRawLogDataForAllCRCs(intervalInDays, eventTypes);
/* 1007 */     List<Map<String, String>> logAttributesForAllCRCs = new ArrayList();
/* 1008 */     for (GenericRecord logRecord : rawLogsForAllCRCs) {
/* 1009 */       if (null != logRecord) {
/* 1010 */         Map<String, String> attrMap = LogEventData.getLogAttributes(logRecord);
/* 1011 */         logAttributesForAllCRCs.add(attrMap);
/*      */       }
/*      */     }
/*      */     
/* 1015 */     return logAttributesForAllCRCs;
/*      */   }
/*      */   
/*      */   public Map<String, Map<String, Map<Long, Map<String, List<String>>>>> getRawPerformanceCounter(String counterObjectName, List<String> instances, List<String> counterNames, List<String> baseCounterNames, int intervalInDays) throws ARMAException
/*      */   {
/* 1020 */     ARMAClientSession session = this.currentarmaObject.getSession();
/* 1021 */     PerfCounters perfData = session.getPerfData(counterObjectName, this.objectName, intervalInDays);
/* 1022 */     Map<String, Map<String, Map<Long, Map<String, List<String>>>>> finalMap = new HashMap();
/* 1023 */     Map<String, Map<Long, Map<String, List<String>>>> instanceMap = null;
/*      */     try {
/* 1025 */       instanceMap = perfData.getCounterDataGroupedByTimestamp(session.getIdentifier(), this.systemId, counterObjectName, counterNames, baseCounterNames, Integer.valueOf(intervalInDays));
/*      */     } catch (ParseException e) {
/* 1027 */       throw new ARMAException(LOG, null, "unable to parse current timestamp : ", e);
/*      */     }
/*      */     
/*      */ 
/* 1031 */     Map<String, Map<Long, Map<String, List<String>>>> finalInstanceMap = null;
/* 1032 */     if (instanceMap != null)
/*      */     {
/* 1034 */       if ((instances != null) && (instances.size() > 0))
/*      */       {
/* 1036 */         finalInstanceMap = new HashMap();
/* 1037 */         for (String instance : instances)
/*      */         {
/* 1039 */           finalInstanceMap.put(instance, instanceMap.get(instance));
/*      */         }
/*      */       }
/*      */       else
/*      */       {
/* 1044 */         finalInstanceMap = instanceMap;
/*      */       }
/* 1046 */       finalMap.put(counterObjectName, finalInstanceMap);
/*      */     }
/* 1048 */     return finalMap;
/*      */   }
/*      */   
/*      */   public MergeType getMergeType()
/*      */   {
/* 1053 */     return this.currentarmaObject.getNodeObject().getMergeType();
/*      */   }
/*      */   
/*      */   public String getInstanceXml() throws ARMAException {
/*      */     try {
/* 1058 */       return ObjectDataXmlHelper.serializeInstance(this);
/*      */     } catch (Exception e) {
/* 1060 */       throw new ARMAException(LOG, "-9999", "unsupported encoding exception occured " + e.getMessage());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   public int hashCode()
/*      */   {
/* 1067 */     StringBuilder key = new StringBuilder();
/* 1068 */     key.append(this.instanceKey.toString());
/* 1069 */     key.append(this.objectName);
/* 1070 */     key.append(this.systemId);
/* 1071 */     return key.hashCode();
/*      */   }
/*      */   
/*      */   public boolean equals(Object obj)
/*      */   {
/* 1076 */     if (this == obj)
/* 1077 */       return true;
/* 1078 */     if (obj == null)
/* 1079 */       return false;
/* 1080 */     if (getClass() != obj.getClass())
/* 1081 */       return false;
/* 1082 */     ARMAObjectInstanceImpl other = (ARMAObjectInstanceImpl)obj;
/* 1083 */     if (this.instanceKey == null) {
/* 1084 */       if (other.instanceKey != null)
/* 1085 */         return false;
/* 1086 */     } else if (!this.instanceKey.equals(other.instanceKey))
/* 1087 */       return false;
/* 1088 */     if (this.objectName == null) {
/* 1089 */       if (other.objectName != null)
/* 1090 */         return false;
/* 1091 */     } else if (!this.objectName.equals(other.objectName))
/* 1092 */       return false;
/* 1093 */     if (this.systemId == null) {
/* 1094 */       if (other.systemId != null)
/* 1095 */         return false;
/* 1096 */     } else if (!this.systemId.equals(other.systemId))
/* 1097 */       return false;
/* 1098 */     return true;
/*      */   }
/*      */   
/*      */   public Map<String, List<GenericRecord>> getCMPerfData(String counterObjectName, int intervalInDays) throws ARMAException
/*      */   {
/* 1103 */     Map<String, List<GenericRecord>> instanceData = new HashMap();
/* 1104 */     ARMAClientSession session = this.currentarmaObject.getSession();
/* 1105 */     PerfCounters perfData = session.getPerfData(this.objectName, counterObjectName, intervalInDays);
/*      */     try
/*      */     {
/* 1108 */       cmPerfCounterData = perfData.getCMPerfCounterData(session.getIdentifier(), this.systemId, counterObjectName, Integer.valueOf(intervalInDays));
/*      */       
/* 1110 */       if ((cmPerfCounterData != null) && (!cmPerfCounterData.isEmpty())) {
/* 1111 */         for (Utf8 instanceName : cmPerfCounterData.keySet())
/* 1112 */           instanceData.put(instanceName.toString(), cmPerfCounterData.get(instanceName));
/*      */       }
/*      */     } catch (ParseException e) {
/*      */       Map<Utf8, GenericData.Array<GenericRecord>> cmPerfCounterData;
/* 1116 */       throw new ARMAException(LOG, null, "unable to parse counter data : ", e);
/*      */     }
/*      */     
/* 1119 */     return instanceData;
/*      */   }
/*      */   
/*      */   public Map<String, List<GenericRecord>> getCMPerfData(String timeStamp, String counterObjectName, int intervalInDays)
/*      */     throws ARMAException
/*      */   {
/* 1125 */     ARMAClientSession session = this.currentarmaObject.getSession();
/* 1126 */     session.getIdentifier().setTimeStamp(timeStamp);
/*      */     
/* 1128 */     return getCMPerfData(counterObjectName, intervalInDays);
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asupdata\client\ARMA\ARMAObjectInstanceImpl.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */