/*     */ package com.sun.xml.bind.v2;
/*     */ 
/*     */ import com.sun.xml.bind.Util;
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.lang.reflect.InvocationTargetException;
/*     */ import java.lang.reflect.Method;
/*     */ import java.lang.reflect.Modifier;
/*     */ import java.util.ArrayList;
/*     */ import java.util.HashSet;
/*     */ import java.util.LinkedList;
/*     */ import java.util.Map;
/*     */ import java.util.Stack;
/*     */ import java.util.TreeSet;
/*     */ import java.util.WeakHashMap;
/*     */ import java.util.logging.Level;
/*     */ import java.util.logging.Logger;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class ClassFactory
/*     */ {
/*  30 */   private static final Class[] emptyClass = new Class[0];
/*  31 */   private static final Object[] emptyObject = new Object[0];
/*     */   
/*  33 */   private static final Logger logger = Util.getClassLogger();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  40 */   private static final ThreadLocal<Map<Class, Constructor>> tls = new ThreadLocal() {
/*     */     public Map<Class, Constructor> initialValue() {
/*  42 */       return new WeakHashMap();
/*     */     }
/*     */   };
/*     */   
/*     */ 
/*     */   public static <T> T create0(Class<T> clazz)
/*     */     throws IllegalAccessException, InvocationTargetException, InstantiationException
/*     */   {
/*  50 */     Map<Class, Constructor> m = (Map)tls.get();
/*  51 */     Constructor<T> cons = (Constructor)m.get(clazz);
/*  52 */     if (cons == null) {
/*     */       try {
/*  54 */         cons = clazz.getDeclaredConstructor(emptyClass);
/*     */       } catch (NoSuchMethodException e) {
/*  56 */         logger.log(Level.INFO, "No default constructor found on " + clazz, e);
/*     */         NoSuchMethodError exp;
/*  58 */         NoSuchMethodError exp; if ((clazz.getDeclaringClass() != null) && (!Modifier.isStatic(clazz.getModifiers()))) {
/*  59 */           exp = new NoSuchMethodError(Messages.NO_DEFAULT_CONSTRUCTOR_IN_INNER_CLASS.format(new Object[] { clazz.getName() }));
/*     */         } else {
/*  61 */           exp = new NoSuchMethodError(e.getMessage());
/*     */         }
/*  63 */         exp.initCause(e);
/*  64 */         throw exp;
/*     */       }
/*     */       
/*  67 */       int classMod = clazz.getModifiers();
/*     */       
/*  69 */       if ((!Modifier.isPublic(classMod)) || (!Modifier.isPublic(cons.getModifiers()))) {
/*     */         try
/*     */         {
/*  72 */           cons.setAccessible(true);
/*     */         }
/*     */         catch (SecurityException e) {
/*  75 */           logger.log(Level.FINE, "Unable to make the constructor of " + clazz + " accessible", e);
/*  76 */           throw e;
/*     */         }
/*     */       }
/*     */       
/*  80 */       m.put(clazz, cons);
/*     */     }
/*     */     
/*  83 */     return (T)cons.newInstance(emptyObject);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public static <T> T create(Class<T> clazz)
/*     */   {
/*     */     try
/*     */     {
/*  92 */       return (T)create0(clazz);
/*     */     } catch (InstantiationException e) {
/*  94 */       logger.log(Level.INFO, "failed to create a new instance of " + clazz, e);
/*  95 */       throw new InstantiationError(e.toString());
/*     */     } catch (IllegalAccessException e) {
/*  97 */       logger.log(Level.INFO, "failed to create a new instance of " + clazz, e);
/*  98 */       throw new IllegalAccessError(e.toString());
/*     */     } catch (InvocationTargetException e) {
/* 100 */       Throwable target = e.getTargetException();
/*     */       
/*     */ 
/*     */ 
/* 104 */       if ((target instanceof RuntimeException)) {
/* 105 */         throw ((RuntimeException)target);
/*     */       }
/*     */       
/* 108 */       if ((target instanceof Error)) {
/* 109 */         throw ((Error)target);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 114 */       throw new IllegalStateException(target);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public static Object create(Method method)
/*     */   {
/* 122 */     Object cons = null;
/* 123 */     Throwable errorMsg = null;
/*     */     try {
/* 125 */       cons = method.invoke(null, emptyObject);
/*     */     } catch (InvocationTargetException ive) {
/* 127 */       Throwable target = ive.getTargetException();
/*     */       
/* 129 */       if ((target instanceof RuntimeException)) {
/* 130 */         throw ((RuntimeException)target);
/*     */       }
/* 132 */       if ((target instanceof Error)) {
/* 133 */         throw ((Error)target);
/*     */       }
/* 135 */       throw new IllegalStateException(target);
/*     */     } catch (IllegalAccessException e) {
/* 137 */       logger.log(Level.INFO, "failed to create a new instance of " + method.getReturnType().getName(), e);
/* 138 */       throw new IllegalAccessError(e.toString());
/*     */     } catch (IllegalArgumentException iae) {
/* 140 */       logger.log(Level.INFO, "failed to create a new instance of " + method.getReturnType().getName(), iae);
/* 141 */       errorMsg = iae;
/*     */     } catch (NullPointerException npe) {
/* 143 */       logger.log(Level.INFO, "failed to create a new instance of " + method.getReturnType().getName(), npe);
/* 144 */       errorMsg = npe;
/*     */     } catch (ExceptionInInitializerError eie) {
/* 146 */       logger.log(Level.INFO, "failed to create a new instance of " + method.getReturnType().getName(), eie);
/* 147 */       errorMsg = eie;
/*     */     }
/* 149 */     if (errorMsg != null)
/*     */     {
/* 151 */       NoSuchMethodError exp = new NoSuchMethodError(errorMsg.getMessage());
/* 152 */       exp.initCause(errorMsg);
/* 153 */       throw exp;
/*     */     }
/* 155 */     return cons;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static <T> Class<? extends T> inferImplClass(Class<T> fieldType, Class[] knownImplClasses)
/*     */   {
/* 165 */     if (!fieldType.isInterface())
/*     */     {
/*     */ 
/* 168 */       return fieldType;
/*     */     }
/* 170 */     for (Class<?> impl : knownImplClasses) {
/* 171 */       if (fieldType.isAssignableFrom(impl)) {
/* 172 */         return impl.asSubclass(fieldType);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 178 */     return null;
/*     */   }
/*     */   
/* 181 */   public static final Class[] COLLECTION_IMPL_CLASSES = { ArrayList.class, LinkedList.class, HashSet.class, TreeSet.class, Stack.class };
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\target\myautosupport\WEB-INF\lib\jaxb-impl-2.0.3.jar!\com\sun\xml\bind\v2\ClassFactory.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */