/*     */ package net.rubyeye.xmemcached.transcoders;
/*     */ 
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.util.Date;
/*     */ import org.slf4j.Logger;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class WhalinV1Transcoder
/*     */   extends BaseSerializingTranscoder
/*     */   implements Transcoder<Object>
/*     */ {
/*     */   public static final int SPECIAL_BYTE = 1;
/*     */   public static final int SPECIAL_BOOLEAN = 2;
/*     */   public static final int SPECIAL_INTEGER = 3;
/*     */   public static final int SPECIAL_LONG = 4;
/*     */   public static final int SPECIAL_CHARACTER = 5;
/*     */   public static final int SPECIAL_STRING = 6;
/*     */   public static final int SPECIAL_STRINGBUFFER = 7;
/*     */   public static final int SPECIAL_FLOAT = 8;
/*     */   public static final int SPECIAL_SHORT = 9;
/*     */   public static final int SPECIAL_DOUBLE = 10;
/*     */   public static final int SPECIAL_DATE = 11;
/*     */   public static final int SPECIAL_STRINGBUILDER = 12;
/*     */   public static final int COMPRESSED = 2;
/*     */   public static final int SERIALIZED = 8;
/*     */   
/*     */   public void setPackZeros(boolean packZeros)
/*     */   {
/*  33 */     throw new UnsupportedOperationException();
/*     */   }
/*     */   
/*     */   public void setPrimitiveAsString(boolean primitiveAsString)
/*     */   {
/*  38 */     throw new UnsupportedOperationException();
/*     */   }
/*     */   
/*  41 */   public boolean isPackZeros() { return false; }
/*     */   
/*     */   public boolean isPrimitiveAsString()
/*     */   {
/*  45 */     return false;
/*     */   }
/*     */   
/*     */   public CachedData encode(Object o)
/*     */   {
/*  50 */     byte[] b = null;
/*  51 */     int flags = 0;
/*  52 */     if ((o instanceof String)) {
/*  53 */       b = encodeW1String((String)o);
/*  54 */     } else if ((o instanceof StringBuffer)) {
/*  55 */       b = encodeStringBuffer((StringBuffer)o);
/*  56 */     } else if ((o instanceof StringBuilder)) {
/*  57 */       b = encodeStringbuilder((StringBuilder)o);
/*  58 */     } else if ((o instanceof Long)) {
/*  59 */       b = encodeLong((Long)o);
/*  60 */     } else if ((o instanceof Integer)) {
/*  61 */       b = encodeInteger((Integer)o);
/*  62 */     } else if ((o instanceof Short)) {
/*  63 */       b = encodeShort((Short)o);
/*  64 */     } else if ((o instanceof Boolean)) {
/*  65 */       b = encodeBoolean((Boolean)o);
/*  66 */     } else if ((o instanceof Date)) {
/*  67 */       b = encodeLong(Long.valueOf(((Date)o).getTime()), 11);
/*  68 */     } else if ((o instanceof Byte)) {
/*  69 */       b = encodeByte((Byte)o);
/*  70 */     } else if ((o instanceof Float)) {
/*  71 */       b = encodeFloat((Float)o);
/*  72 */     } else if ((o instanceof Double)) {
/*  73 */       b = encodeDouble((Double)o);
/*  74 */     } else { if ((o instanceof byte[]))
/*  75 */         throw new IllegalArgumentException("Cannot handle byte arrays.");
/*  76 */       if ((o instanceof Character)) {
/*  77 */         b = encodeCharacter((Character)o);
/*     */       } else {
/*  79 */         b = serialize(o);
/*  80 */         flags |= 0x8;
/*     */       } }
/*  82 */     assert (b != null);
/*  83 */     if (b.length > this.compressionThreshold) {
/*  84 */       byte[] compressed = compress(b);
/*  85 */       if (compressed.length < b.length) {
/*  86 */         log.debug(String.format("Compressed %s from %d to %d", new Object[] { o.getClass().getName(), Integer.valueOf(b.length), Integer.valueOf(compressed.length) }));
/*     */         
/*  88 */         b = compressed;
/*  89 */         flags |= 0x2;
/*     */       } else {
/*  91 */         log.debug(String.format("Compression increased the size of %s from %d to %d", new Object[] { o.getClass().getName(), Integer.valueOf(b.length), Integer.valueOf(compressed.length) }));
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*  97 */     return new CachedData(flags, b);
/*     */   }
/*     */   
/*     */   public Object decode(CachedData d) {
/* 101 */     byte[] data = d.getData();
/* 102 */     Object rv = null;
/* 103 */     if ((d.getFlag() & 0x2) != 0) {
/* 104 */       data = decompress(d.getData());
/*     */     }
/* 106 */     if ((d.getFlag() & 0x8) != 0) {
/* 107 */       rv = deserialize(data);
/*     */     } else {
/* 109 */       int f = data[0];
/* 110 */       switch (f) {
/*     */       case 2: 
/* 112 */         rv = decodeBoolean(data);
/* 113 */         break;
/*     */       case 3: 
/* 115 */         rv = decodeInteger(data);
/* 116 */         break;
/*     */       case 9: 
/* 118 */         rv = decodeShort(data);
/* 119 */         break;
/*     */       case 4: 
/* 121 */         rv = decodeLong(data);
/* 122 */         break;
/*     */       case 11: 
/* 124 */         rv = new Date(decodeLong(data).longValue());
/* 125 */         break;
/*     */       case 1: 
/* 127 */         rv = decodeByte(data);
/* 128 */         break;
/*     */       case 8: 
/* 130 */         rv = decodeFloat(data);
/* 131 */         break;
/*     */       case 10: 
/* 133 */         rv = decodeDouble(data);
/* 134 */         break;
/*     */       case 6: 
/* 136 */         rv = decodeW1String(data);
/* 137 */         break;
/*     */       case 7: 
/* 139 */         rv = new StringBuffer(decodeW1String(data));
/* 140 */         break;
/*     */       case 12: 
/* 142 */         rv = new StringBuilder(decodeW1String(data));
/* 143 */         break;
/*     */       case 5: 
/* 145 */         rv = decodeCharacter(data);
/* 146 */         break;
/*     */       default: 
/* 148 */         log.warn(String.format("Cannot handle data with flags %x", new Object[] { Integer.valueOf(f) }));
/*     */       }
/*     */     }
/* 151 */     return rv;
/*     */   }
/*     */   
/*     */   private Short decodeShort(byte[] data) {
/* 155 */     return Short.valueOf((short)decodeInteger(data).intValue());
/*     */   }
/*     */   
/*     */   private Byte decodeByte(byte[] in) {
/* 159 */     assert (in.length == 2) : "Wrong length for a byte";
/* 160 */     byte value = in[1];
/* 161 */     return Byte.valueOf(value);
/*     */   }
/*     */   
/*     */   private Integer decodeInteger(byte[] in)
/*     */   {
/* 166 */     assert (in.length == 5) : "Wrong length for an int";
/* 167 */     return Integer.valueOf((int)decodeLong(in).longValue());
/*     */   }
/*     */   
/*     */   private Float decodeFloat(byte[] in)
/*     */   {
/* 172 */     assert (in.length == 5) : "Wrong length for a float";
/* 173 */     Integer l = decodeInteger(in);
/* 174 */     return Float.valueOf(Float.intBitsToFloat(l.intValue()));
/*     */   }
/*     */   
/*     */   private Double decodeDouble(byte[] in) {
/* 178 */     assert (in.length == 9) : "Wrong length for a double";
/* 179 */     Long l = decodeLong(in);
/* 180 */     return Double.valueOf(Double.longBitsToDouble(l.longValue()));
/*     */   }
/*     */   
/*     */   private Boolean decodeBoolean(byte[] in) {
/* 184 */     assert (in.length == 2) : "Wrong length for a boolean";
/* 185 */     return Boolean.valueOf(in[1] == 1);
/*     */   }
/*     */   
/*     */   private Long decodeLong(byte[] in) {
/* 189 */     long rv = 0L;
/* 190 */     for (int idx = 1; idx < in.length; idx++) {
/* 191 */       byte i = in[idx];
/* 192 */       rv = rv << 8 | (i < 0 ? 256 + i : i);
/*     */     }
/* 194 */     return Long.valueOf(rv);
/*     */   }
/*     */   
/*     */   private Character decodeCharacter(byte[] b) {
/* 198 */     return Character.valueOf((char)decodeInteger(b).intValue());
/*     */   }
/*     */   
/*     */   private String decodeW1String(byte[] b) {
/*     */     try {
/* 203 */       return new String(b, 1, b.length - 1, this.charset);
/*     */     } catch (UnsupportedEncodingException e) {
/* 205 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   private byte[] encodeByte(Byte value) {
/* 210 */     byte[] b = new byte[2];
/* 211 */     b[0] = 1;
/* 212 */     b[1] = value.byteValue();
/* 213 */     return b;
/*     */   }
/*     */   
/*     */   private byte[] encodeBoolean(Boolean value) {
/* 217 */     byte[] b = new byte[2];
/* 218 */     b[0] = 2;
/* 219 */     b[1] = ((byte)(value.booleanValue() ? 1 : 0));
/* 220 */     return b;
/*     */   }
/*     */   
/*     */   private byte[] encodeInteger(Integer value) {
/* 224 */     byte[] b = encodeNum(value.intValue(), 4);
/* 225 */     b[0] = 3;
/* 226 */     return b;
/*     */   }
/*     */   
/*     */   private byte[] encodeLong(Long value, int type) {
/* 230 */     byte[] b = encodeNum(value.longValue(), 8);
/* 231 */     b[0] = ((byte)type);
/* 232 */     return b;
/*     */   }
/*     */   
/*     */   private byte[] encodeLong(Long value) {
/* 236 */     return encodeLong(value, 4);
/*     */   }
/*     */   
/*     */   private byte[] encodeShort(Short value) {
/* 240 */     byte[] b = encodeInteger(Integer.valueOf(value.shortValue()));
/* 241 */     b[0] = 9;
/* 242 */     return b;
/*     */   }
/*     */   
/*     */   private byte[] encodeFloat(Float value) {
/* 246 */     byte[] b = encodeInteger(Integer.valueOf(Float.floatToIntBits(value.floatValue())));
/* 247 */     b[0] = 8;
/* 248 */     return b;
/*     */   }
/*     */   
/*     */   private byte[] encodeDouble(Double value) {
/* 252 */     byte[] b = encodeLong(Long.valueOf(Double.doubleToLongBits(value.doubleValue())));
/* 253 */     b[0] = 10;
/* 254 */     return b;
/*     */   }
/*     */   
/*     */   private byte[] encodeCharacter(Character value) {
/* 258 */     byte[] result = encodeInteger(Integer.valueOf(value.charValue()));
/* 259 */     result[0] = 5;
/* 260 */     return result;
/*     */   }
/*     */   
/*     */   private byte[] encodeStringBuffer(StringBuffer value) {
/* 264 */     byte[] b = encodeW1String(value.toString());
/* 265 */     b[0] = 7;
/* 266 */     return b;
/*     */   }
/*     */   
/*     */   private byte[] encodeStringbuilder(StringBuilder value) {
/* 270 */     byte[] b = encodeW1String(value.toString());
/* 271 */     b[0] = 12;
/* 272 */     return b;
/*     */   }
/*     */   
/*     */   private byte[] encodeW1String(String value) {
/* 276 */     byte[] svalue = null;
/*     */     try {
/* 278 */       svalue = value.getBytes(this.charset);
/*     */     } catch (UnsupportedEncodingException e) {
/* 280 */       throw new RuntimeException(e);
/*     */     }
/* 282 */     byte[] result = new byte[svalue.length + 1];
/* 283 */     System.arraycopy(svalue, 0, result, 1, svalue.length);
/* 284 */     result[0] = 6;
/* 285 */     return result;
/*     */   }
/*     */   
/*     */   private byte[] encodeNum(long l, int maxBytes) {
/* 289 */     byte[] rv = new byte[maxBytes + 1];
/*     */     
/* 291 */     for (int i = 0; i < rv.length - 1; i++) {
/* 292 */       int pos = rv.length - i - 1;
/* 293 */       rv[pos] = ((byte)(int)(l >> 8 * i & 0xFF));
/*     */     }
/*     */     
/* 296 */     return rv;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\transcoders\WhalinV1Transcoder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */