/*      */ package org.apache.avro;
/*      */ 
/*      */ import java.io.File;
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.io.StringReader;
/*      */ import java.io.StringWriter;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Collection;
/*      */ import java.util.Collections;
/*      */ import java.util.HashMap;
/*      */ import java.util.HashSet;
/*      */ import java.util.IdentityHashMap;
/*      */ import java.util.Iterator;
/*      */ import java.util.LinkedHashMap;
/*      */ import java.util.LinkedHashSet;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Map.Entry;
/*      */ import java.util.Set;
/*      */ import org.codehaus.jackson.JsonFactory;
/*      */ import org.codehaus.jackson.JsonGenerator;
/*      */ import org.codehaus.jackson.JsonNode;
/*      */ import org.codehaus.jackson.JsonParseException;
/*      */ import org.codehaus.jackson.JsonParser;
/*      */ import org.codehaus.jackson.JsonParser.Feature;
/*      */ import org.codehaus.jackson.map.ObjectMapper;
/*      */ import org.codehaus.jackson.node.DoubleNode;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public abstract class Schema
/*      */ {
/*   79 */   static final JsonFactory FACTORY = new JsonFactory();
/*   80 */   static final ObjectMapper MAPPER = new ObjectMapper(FACTORY);
/*      */   
/*      */   private static final int NO_HASHCODE = Integer.MIN_VALUE;
/*      */   
/*      */   private final Type type;
/*      */   private static final Set<String> SCHEMA_RESERVED;
/*      */   
/*      */   public static enum Type
/*      */   {
/*      */     private Type() {}
/*      */     
/*   91 */     RECORD,  ENUM,  ARRAY,  MAP,  UNION,  FIXED,  STRING,  BYTES, 
/*   92 */     INT,  LONG,  FLOAT,  DOUBLE,  BOOLEAN,  NULL;
/*      */     
/*   94 */     private String name = name().toLowerCase();
/*   95 */     public String getName() { return this.name; }
/*      */   }
/*      */   
/*      */   Schema(Type type)
/*      */   {
/*  100 */     this.type = type;
/*      */   }
/*      */   
/*      */   public static Schema create(Type type) {
/*  104 */     switch (type) {
/*  105 */     case STRING:  return new StringSchema();
/*  106 */     case BYTES:  return new BytesSchema();
/*  107 */     case INT:  return new IntSchema();
/*  108 */     case LONG:  return new LongSchema();
/*  109 */     case FLOAT:  return new FloatSchema();
/*  110 */     case DOUBLE:  return new DoubleSchema();
/*  111 */     case BOOLEAN:  return new BooleanSchema();
/*  112 */     case NULL:  return new NullSchema(); }
/*  113 */     throw new AvroRuntimeException("Can't create a: " + type);
/*      */   }
/*      */   
/*      */   static final class Props extends LinkedHashMap<String, String> {
/*      */     private Set<String> reserved;
/*      */     
/*      */     public Props(Set<String> reserved) {
/*  120 */       super();
/*  121 */       this.reserved = reserved;
/*      */     }
/*      */     
/*  124 */     public void add(String name, String value) { if (this.reserved.contains(name)) {
/*  125 */         throw new AvroRuntimeException("Can't set reserved property: " + name);
/*      */       }
/*  127 */       if (value == null) {
/*  128 */         throw new AvroRuntimeException("Can't set a property to null: " + name);
/*      */       }
/*  130 */       String old = (String)get(name);
/*  131 */       if (old == null) {
/*  132 */         put(name, value);
/*  133 */       } else if (!old.equals(value))
/*  134 */         throw new AvroRuntimeException("Can't overwrite property: " + name);
/*      */     }
/*      */     
/*      */     public void write(JsonGenerator gen) throws IOException {
/*  138 */       for (Map.Entry<String, String> e : entrySet()) {
/*  139 */         gen.writeStringField((String)e.getKey(), (String)e.getValue());
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  150 */   Props props = new Props(SCHEMA_RESERVED);
/*  151 */   int hashCode = Integer.MIN_VALUE;
/*      */   private static final Set<String> FIELD_RESERVED;
/*      */   private static final ThreadLocal<Set> SEEN_EQUALS;
/*      */   private static final ThreadLocal<Map> SEEN_HASHCODE;
/*      */   static final Map<String, Type> PRIMITIVES;
/*      */   
/*      */   public synchronized String getProp(String name) {
/*  158 */     return (String)this.props.get(name);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public synchronized void addProp(String name, String value)
/*      */   {
/*  171 */     this.props.add(name, value);
/*  172 */     this.hashCode = Integer.MIN_VALUE;
/*      */   }
/*      */   
/*      */   public Map<String, String> getProps()
/*      */   {
/*  177 */     return Collections.unmodifiableMap(this.props);
/*      */   }
/*      */   
/*      */   public static Schema createRecord(List<Field> fields)
/*      */   {
/*  182 */     Schema result = createRecord(null, null, null, false);
/*  183 */     result.setFields(fields);
/*  184 */     return result;
/*      */   }
/*      */   
/*      */ 
/*      */   public static Schema createRecord(String name, String doc, String namespace, boolean isError)
/*      */   {
/*  190 */     return new RecordSchema(new Name(name, namespace), doc, isError);
/*      */   }
/*      */   
/*      */ 
/*      */   public static Schema createEnum(String name, String doc, String namespace, List<String> values)
/*      */   {
/*  196 */     return new EnumSchema(new Name(name, namespace), doc, new LockableArrayList(values));
/*      */   }
/*      */   
/*      */ 
/*      */   public static Schema createArray(Schema elementType)
/*      */   {
/*  202 */     return new ArraySchema(elementType);
/*      */   }
/*      */   
/*      */   public static Schema createMap(Schema valueType)
/*      */   {
/*  207 */     return new MapSchema(valueType);
/*      */   }
/*      */   
/*      */   public static Schema createUnion(List<Schema> types)
/*      */   {
/*  212 */     return new UnionSchema(new LockableArrayList(types));
/*      */   }
/*      */   
/*      */ 
/*      */   public static Schema createFixed(String name, String doc, String space, int size)
/*      */   {
/*  218 */     return new FixedSchema(new Name(name, space), doc, size);
/*      */   }
/*      */   
/*      */   public Type getType() {
/*  222 */     return this.type;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Field getField(String fieldname)
/*      */   {
/*  230 */     throw new AvroRuntimeException("Not a record: " + this);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public List<Field> getFields()
/*      */   {
/*  238 */     throw new AvroRuntimeException("Not a record: " + this);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setFields(List<Field> fields)
/*      */   {
/*  246 */     throw new AvroRuntimeException("Not a record: " + this);
/*      */   }
/*      */   
/*      */   public List<String> getEnumSymbols()
/*      */   {
/*  251 */     throw new AvroRuntimeException("Not an enum: " + this);
/*      */   }
/*      */   
/*      */   public int getEnumOrdinal(String symbol)
/*      */   {
/*  256 */     throw new AvroRuntimeException("Not an enum: " + this);
/*      */   }
/*      */   
/*      */   public boolean hasEnumSymbol(String symbol)
/*      */   {
/*  261 */     throw new AvroRuntimeException("Not an enum: " + this);
/*      */   }
/*      */   
/*      */   public String getName()
/*      */   {
/*  266 */     return this.type.name;
/*      */   }
/*      */   
/*      */   public String getDoc()
/*      */   {
/*  271 */     return null;
/*      */   }
/*      */   
/*      */   public String getNamespace()
/*      */   {
/*  276 */     throw new AvroRuntimeException("Not a named type: " + this);
/*      */   }
/*      */   
/*      */ 
/*      */   public String getFullName()
/*      */   {
/*  282 */     return getName();
/*      */   }
/*      */   
/*      */   public void addAlias(String alias)
/*      */   {
/*  287 */     throw new AvroRuntimeException("Not a named type: " + this);
/*      */   }
/*      */   
/*      */   public Set<String> getAliases()
/*      */   {
/*  292 */     throw new AvroRuntimeException("Not a named type: " + this);
/*      */   }
/*      */   
/*      */   public boolean isError()
/*      */   {
/*  297 */     throw new AvroRuntimeException("Not a record: " + this);
/*      */   }
/*      */   
/*      */   public Schema getElementType()
/*      */   {
/*  302 */     throw new AvroRuntimeException("Not an array: " + this);
/*      */   }
/*      */   
/*      */   public Schema getValueType()
/*      */   {
/*  307 */     throw new AvroRuntimeException("Not a map: " + this);
/*      */   }
/*      */   
/*      */   public List<Schema> getTypes()
/*      */   {
/*  312 */     throw new AvroRuntimeException("Not a union: " + this);
/*      */   }
/*      */   
/*      */   public Integer getIndexNamed(String name)
/*      */   {
/*  317 */     throw new AvroRuntimeException("Not a union: " + this);
/*      */   }
/*      */   
/*      */   public int getFixedSize()
/*      */   {
/*  322 */     throw new AvroRuntimeException("Not fixed: " + this);
/*      */   }
/*      */   
/*      */   public String toString()
/*      */   {
/*  327 */     return toString(false);
/*      */   }
/*      */   
/*      */   public String toString(boolean pretty)
/*      */   {
/*      */     try
/*      */     {
/*  334 */       StringWriter writer = new StringWriter();
/*  335 */       JsonGenerator gen = FACTORY.createJsonGenerator(writer);
/*  336 */       if (pretty) gen.useDefaultPrettyPrinter();
/*  337 */       toJson(new Names(), gen);
/*  338 */       gen.flush();
/*  339 */       return writer.toString();
/*      */     } catch (IOException e) {
/*  341 */       throw new AvroRuntimeException(e);
/*      */     }
/*      */   }
/*      */   
/*      */   void toJson(Names names, JsonGenerator gen) throws IOException {
/*  346 */     if (this.props.size() == 0) {
/*  347 */       gen.writeString(getName());
/*      */     } else {
/*  349 */       gen.writeStartObject();
/*  350 */       gen.writeStringField("type", getName());
/*  351 */       this.props.write(gen);
/*  352 */       gen.writeEndObject();
/*      */     }
/*      */   }
/*      */   
/*      */   void fieldsToJson(Names names, JsonGenerator gen) throws IOException {
/*  357 */     throw new AvroRuntimeException("Not a record: " + this);
/*      */   }
/*      */   
/*      */   public boolean equals(Object o) {
/*  361 */     if (o == this) return true;
/*  362 */     if (!(o instanceof Schema)) return false;
/*  363 */     Schema that = (Schema)o;
/*  364 */     if (this.type != that.type) return false;
/*  365 */     return (equalCachedHash(that)) && (this.props.equals(that.props));
/*      */   }
/*      */   
/*  368 */   public final int hashCode() { if (this.hashCode == Integer.MIN_VALUE)
/*  369 */       this.hashCode = computeHash();
/*  370 */     return this.hashCode;
/*      */   }
/*      */   
/*  373 */   int computeHash() { return getType().hashCode() + this.props.hashCode(); }
/*      */   
/*      */   final boolean equalCachedHash(Schema other) {
/*  376 */     return (this.hashCode == other.hashCode) || (this.hashCode == Integer.MIN_VALUE) || (other.hashCode == Integer.MIN_VALUE);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public static class Field
/*      */   {
/*      */     private final String name;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     public static enum Order
/*      */     {
/*  391 */       ASCENDING,  DESCENDING,  IGNORE;
/*      */       
/*  393 */       private String name = name().toLowerCase();
/*      */       
/*      */       private Order() {} }
/*      */     
/*  397 */     private transient int position = -1;
/*      */     private final Schema schema;
/*      */     private final String doc;
/*      */     private final JsonNode defaultValue;
/*      */     private final Order order;
/*      */     private Set<String> aliases;
/*  403 */     private final Schema.Props props = new Schema.Props(Schema.FIELD_RESERVED);
/*      */     
/*      */     public Field(String name, Schema schema, String doc, JsonNode defaultValue)
/*      */     {
/*  407 */       this(name, schema, doc, defaultValue, Order.ASCENDING);
/*      */     }
/*      */     
/*      */     public Field(String name, Schema schema, String doc, JsonNode defaultValue, Order order) {
/*  411 */       this.name = Schema.validateName(name);
/*  412 */       this.schema = schema;
/*  413 */       this.doc = doc;
/*  414 */       this.defaultValue = defaultValue;
/*  415 */       this.order = order; }
/*      */     
/*  417 */     public String name() { return this.name; }
/*      */     
/*  419 */     public int pos() { return this.position; }
/*      */     
/*  421 */     public Schema schema() { return this.schema; }
/*      */     
/*  423 */     public String doc() { return this.doc; }
/*  424 */     public JsonNode defaultValue() { return this.defaultValue; }
/*  425 */     public Order order() { return this.order; }
/*      */     
/*  427 */     public synchronized String getProp(String name) { return (String)this.props.get(name); }
/*      */     
/*      */     public synchronized void addProp(String name, String value) {
/*  430 */       this.props.add(name, value);
/*      */     }
/*      */     
/*      */ 
/*  434 */     public Map<String, String> props() { return Collections.unmodifiableMap(this.props); }
/*      */     
/*      */     public void addAlias(String alias) {
/*  437 */       if (this.aliases == null)
/*  438 */         this.aliases = new LinkedHashSet();
/*  439 */       this.aliases.add(alias);
/*      */     }
/*      */     
/*      */     public Set<String> aliases() {
/*  443 */       if (this.aliases == null)
/*  444 */         return Collections.emptySet();
/*  445 */       return Collections.unmodifiableSet(this.aliases);
/*      */     }
/*      */     
/*  448 */     public boolean equals(Object other) { if (other == this) return true;
/*  449 */       if (!(other instanceof Field)) return false;
/*  450 */       Field that = (Field)other;
/*  451 */       return (this.name.equals(that.name)) && (this.schema.equals(that.schema)) && (defaultValueEquals(that.defaultValue)) && (this.order == that.order) && (this.props.equals(that.props));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  457 */     public int hashCode() { return this.name.hashCode() + this.schema.computeHash(); }
/*      */     
/*      */     private boolean defaultValueEquals(JsonNode thatDefaultValue) {
/*  460 */       if (this.defaultValue == null)
/*  461 */         return thatDefaultValue == null;
/*  462 */       if (Double.isNaN(this.defaultValue.getDoubleValue()))
/*  463 */         return Double.isNaN(thatDefaultValue.getDoubleValue());
/*  464 */       return this.defaultValue.equals(thatDefaultValue);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  469 */     public String toString() { return this.name + " type:" + this.schema.type + " pos:" + this.position; }
/*      */   }
/*      */   
/*      */   private static class Name {
/*      */     private final String name;
/*      */     private final String space;
/*      */     private final String full;
/*      */     
/*      */     public Name(String name, String space) {
/*  478 */       if (name == null) {
/*  479 */         this.name = (this.space = this.full = null);
/*  480 */         return;
/*      */       }
/*  482 */       int lastDot = name.lastIndexOf('.');
/*  483 */       if (lastDot < 0) {
/*  484 */         this.space = space;
/*  485 */         this.name = Schema.validateName(name);
/*      */       } else {
/*  487 */         this.space = name.substring(0, lastDot);
/*  488 */         this.name = Schema.validateName(name.substring(lastDot + 1, name.length()));
/*      */       }
/*  490 */       this.full = (this.space + "." + this.name);
/*      */     }
/*      */     
/*  493 */     public boolean equals(Object o) { if (o == this) return true;
/*  494 */       if (!(o instanceof Name)) return false;
/*  495 */       Name that = (Name)o;
/*  496 */       return this.full == null ? false : that.full == null ? true : this.full.equals(that.full);
/*      */     }
/*      */     
/*  499 */     public int hashCode() { return this.full == null ? 0 : this.full.hashCode(); }
/*      */     
/*  501 */     public String toString() { return this.full; }
/*      */     
/*  503 */     public void writeName(Schema.Names names, JsonGenerator gen) throws IOException { if (this.name != null) gen.writeStringField("name", this.name);
/*  504 */       if (this.space != null) {
/*  505 */         if (!this.space.equals(names.space()))
/*  506 */           gen.writeStringField("namespace", this.space);
/*  507 */         if (names.space() == null)
/*  508 */           names.space(this.space);
/*      */       }
/*      */     }
/*      */     
/*  512 */     public String getQualified(String defaultSpace) { return (this.space == null) || (this.space.equals(defaultSpace)) ? this.name : this.full; }
/*      */   }
/*      */   
/*      */   private static abstract class NamedSchema extends Schema {
/*      */     final Schema.Name name;
/*      */     final String doc;
/*      */     Set<Schema.Name> aliases;
/*      */     
/*      */     public NamedSchema(Schema.Type type, Schema.Name name, String doc) {
/*  521 */       super();
/*  522 */       this.name = name;
/*  523 */       this.doc = doc;
/*  524 */       if (PRIMITIVES.containsKey(Schema.Name.access$400(name)))
/*  525 */         throw new AvroTypeException("Schemas may not be named after primitives: " + Schema.Name.access$400(name));
/*      */     }
/*      */     
/*  528 */     public String getName() { return Schema.Name.access$500(this.name); }
/*  529 */     public String getDoc() { return this.doc; }
/*  530 */     public String getNamespace() { return Schema.Name.access$600(this.name); }
/*  531 */     public String getFullName() { return Schema.Name.access$400(this.name); }
/*      */     
/*  533 */     public void addAlias(String alias) { if (this.aliases == null)
/*  534 */         this.aliases = new LinkedHashSet();
/*  535 */       this.aliases.add(new Schema.Name(alias, Schema.Name.access$600(this.name)));
/*      */     }
/*      */     
/*  538 */     public Set<String> getAliases() { Set<String> result = new LinkedHashSet();
/*  539 */       if (this.aliases != null)
/*  540 */         for (Schema.Name alias : this.aliases)
/*  541 */           result.add(Schema.Name.access$400(alias));
/*  542 */       return result;
/*      */     }
/*      */     
/*      */     public boolean writeNameRef(Schema.Names names, JsonGenerator gen) throws IOException {
/*  546 */       if (equals(names.get(this.name))) {
/*  547 */         gen.writeString(this.name.getQualified(names.space()));
/*  548 */         return true; }
/*  549 */       if (Schema.Name.access$500(this.name) != null) {
/*  550 */         names.put(this.name, this);
/*      */       }
/*  552 */       return false;
/*      */     }
/*      */     
/*  555 */     public void writeName(Schema.Names names, JsonGenerator gen) throws IOException { this.name.writeName(names, gen); }
/*      */     
/*      */     public boolean equalNames(NamedSchema that) {
/*  558 */       return this.name.equals(that.name);
/*      */     }
/*      */     
/*  561 */     int computeHash() { return super.computeHash() + this.name.hashCode(); }
/*      */     
/*      */     public void aliasesToJson(JsonGenerator gen) throws IOException {
/*  564 */       if ((this.aliases == null) || (this.aliases.size() == 0)) return;
/*  565 */       gen.writeFieldName("aliases");
/*  566 */       gen.writeStartArray();
/*  567 */       for (Schema.Name alias : this.aliases)
/*  568 */         gen.writeString(alias.getQualified(Schema.Name.access$600(this.name)));
/*  569 */       gen.writeEndArray();
/*      */     }
/*      */   }
/*      */   
/*      */   private static class SeenPair { private Object s1;
/*      */     private Object s2;
/*      */     
/*  576 */     private SeenPair(Object s1, Object s2) { this.s1 = s1;this.s2 = s2; }
/*      */     
/*  578 */     public boolean equals(Object o) { return (this.s1 == ((SeenPair)o).s1) && (this.s2 == ((SeenPair)o).s2); }
/*      */     
/*      */     public int hashCode() {
/*  581 */       return System.identityHashCode(this.s1) + System.identityHashCode(this.s2);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   private static class RecordSchema
/*      */     extends Schema.NamedSchema
/*      */   {
/*      */     private List<Schema.Field> fields;
/*      */     
/*      */     private Map<String, Schema.Field> fieldMap;
/*      */     
/*      */     private final boolean isError;
/*      */     
/*      */ 
/*      */     public RecordSchema(Schema.Name name, String doc, boolean isError)
/*      */     {
/*  598 */       super(name, doc);
/*  599 */       this.isError = isError; }
/*      */     
/*  601 */     public boolean isError() { return this.isError; }
/*      */     
/*      */     public Schema.Field getField(String fieldname)
/*      */     {
/*  605 */       if (this.fieldMap == null)
/*  606 */         throw new AvroRuntimeException("Schema fields not set yet");
/*  607 */       return (Schema.Field)this.fieldMap.get(fieldname);
/*      */     }
/*      */     
/*      */     public List<Schema.Field> getFields()
/*      */     {
/*  612 */       if (this.fields == null)
/*  613 */         throw new AvroRuntimeException("Schema fields not set yet");
/*  614 */       return this.fields;
/*      */     }
/*      */     
/*      */     public void setFields(List<Schema.Field> fields)
/*      */     {
/*  619 */       if (this.fields != null) {
/*  620 */         throw new AvroRuntimeException("Fields are already set");
/*      */       }
/*  622 */       int i = 0;
/*  623 */       this.fieldMap = new HashMap();
/*  624 */       Schema.LockableArrayList ff = new Schema.LockableArrayList();
/*  625 */       for (Schema.Field f : fields) {
/*  626 */         if (Schema.Field.access$700(f) != -1)
/*  627 */           throw new AvroRuntimeException("Field already used: " + f);
/*  628 */         Schema.Field.access$702(f, i++);
/*  629 */         this.fieldMap.put(f.name(), f);
/*  630 */         ff.add(f);
/*      */       }
/*  632 */       this.fields = ff.lock();
/*  633 */       this.hashCode = Integer.MIN_VALUE;
/*      */     }
/*      */     
/*  636 */     public boolean equals(Object o) { if (o == this) return true;
/*  637 */       if (!(o instanceof RecordSchema)) return false;
/*  638 */       RecordSchema that = (RecordSchema)o;
/*  639 */       if (!equalCachedHash(that)) return false;
/*  640 */       if (!equalNames(that)) return false;
/*  641 */       if (!this.props.equals(that.props)) return false;
/*  642 */       Set seen = (Set)Schema.SEEN_EQUALS.get();
/*  643 */       Schema.SeenPair here = new Schema.SeenPair(this, o, null);
/*  644 */       if (seen.contains(here)) return true;
/*  645 */       boolean first = seen.isEmpty();
/*      */       try {
/*  647 */         seen.add(here);
/*  648 */         return this.fields.equals(((RecordSchema)o).fields);
/*      */       } finally {
/*  650 */         if (first) seen.clear();
/*      */       }
/*      */     }
/*      */     
/*  654 */     int computeHash() { Map seen = (Map)Schema.SEEN_HASHCODE.get();
/*  655 */       if (seen.containsKey(this)) return 0;
/*  656 */       boolean first = seen.isEmpty();
/*      */       try {
/*  658 */         seen.put(this, this);
/*  659 */         return super.computeHash() + this.fields.hashCode();
/*      */       } finally {
/*  661 */         if (first) seen.clear();
/*      */       }
/*      */     }
/*      */     
/*  665 */     void toJson(Schema.Names names, JsonGenerator gen) throws IOException { if (writeNameRef(names, gen)) return;
/*  666 */       String savedSpace = names.space;
/*  667 */       gen.writeStartObject();
/*  668 */       gen.writeStringField("type", this.isError ? "error" : "record");
/*  669 */       writeName(names, gen);
/*  670 */       names.space = Schema.Name.access$600(this.name);
/*  671 */       if (getDoc() != null)
/*  672 */         gen.writeStringField("doc", getDoc());
/*  673 */       gen.writeFieldName("fields");
/*  674 */       fieldsToJson(names, gen);
/*  675 */       this.props.write(gen);
/*  676 */       aliasesToJson(gen);
/*  677 */       gen.writeEndObject();
/*  678 */       names.space = savedSpace;
/*      */     }
/*      */     
/*      */     void fieldsToJson(Schema.Names names, JsonGenerator gen) throws IOException {
/*  682 */       gen.writeStartArray();
/*  683 */       for (Schema.Field f : this.fields) {
/*  684 */         gen.writeStartObject();
/*  685 */         gen.writeStringField("name", f.name());
/*  686 */         gen.writeFieldName("type");
/*  687 */         f.schema().toJson(names, gen);
/*  688 */         if (f.doc() != null)
/*  689 */           gen.writeStringField("doc", f.doc());
/*  690 */         if (f.defaultValue() != null) {
/*  691 */           gen.writeFieldName("default");
/*  692 */           gen.writeTree(f.defaultValue());
/*      */         }
/*  694 */         if (f.order() != Schema.Field.Order.ASCENDING)
/*  695 */           gen.writeStringField("order", Schema.Field.Order.access$1200(f.order()));
/*  696 */         if ((Schema.Field.access$1300(f) != null) && (Schema.Field.access$1300(f).size() != 0)) {
/*  697 */           gen.writeFieldName("aliases");
/*  698 */           gen.writeStartArray();
/*  699 */           for (String alias : Schema.Field.access$1300(f))
/*  700 */             gen.writeString(alias);
/*  701 */           gen.writeEndArray();
/*      */         }
/*  703 */         Schema.Field.access$1400(f).write(gen);
/*  704 */         gen.writeEndObject();
/*      */       }
/*  706 */       gen.writeEndArray();
/*      */     }
/*      */   }
/*      */   
/*      */   private static class EnumSchema extends Schema.NamedSchema {
/*      */     private final List<String> symbols;
/*      */     private final Map<String, Integer> ordinals;
/*      */     
/*      */     public EnumSchema(Schema.Name name, String doc, Schema.LockableArrayList<String> symbols) {
/*  715 */       super(name, doc);
/*  716 */       this.symbols = symbols.lock();
/*  717 */       this.ordinals = new HashMap();
/*  718 */       int i = 0;
/*  719 */       for (String symbol : symbols)
/*  720 */         if (this.ordinals.put(Schema.validateName(symbol), Integer.valueOf(i++)) != null)
/*  721 */           throw new SchemaParseException("Duplicate enum symbol: " + symbol); }
/*      */     
/*  723 */     public List<String> getEnumSymbols() { return this.symbols; }
/*      */     
/*  725 */     public boolean hasEnumSymbol(String symbol) { return this.ordinals.containsKey(symbol); }
/*  726 */     public int getEnumOrdinal(String symbol) { return ((Integer)this.ordinals.get(symbol)).intValue(); }
/*      */     
/*  728 */     public boolean equals(Object o) { if (o == this) return true;
/*  729 */       if (!(o instanceof EnumSchema)) return false;
/*  730 */       EnumSchema that = (EnumSchema)o;
/*  731 */       return (equalCachedHash(that)) && (equalNames(that)) && (this.symbols.equals(that.symbols)) && (this.props.equals(that.props));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  736 */     int computeHash() { return super.computeHash() + this.symbols.hashCode(); }
/*      */     
/*  738 */     void toJson(Schema.Names names, JsonGenerator gen) throws IOException { if (writeNameRef(names, gen)) return;
/*  739 */       gen.writeStartObject();
/*  740 */       gen.writeStringField("type", "enum");
/*  741 */       writeName(names, gen);
/*  742 */       if (getDoc() != null)
/*  743 */         gen.writeStringField("doc", getDoc());
/*  744 */       gen.writeArrayFieldStart("symbols");
/*  745 */       for (String symbol : this.symbols)
/*  746 */         gen.writeString(symbol);
/*  747 */       gen.writeEndArray();
/*  748 */       this.props.write(gen);
/*  749 */       aliasesToJson(gen);
/*  750 */       gen.writeEndObject();
/*      */     }
/*      */   }
/*      */   
/*      */   private static class ArraySchema extends Schema {
/*      */     private final Schema elementType;
/*      */     
/*  757 */     public ArraySchema(Schema elementType) { super();
/*  758 */       this.elementType = elementType; }
/*      */     
/*  760 */     public Schema getElementType() { return this.elementType; }
/*      */     
/*  762 */     public boolean equals(Object o) { if (o == this) return true;
/*  763 */       if (!(o instanceof ArraySchema)) return false;
/*  764 */       ArraySchema that = (ArraySchema)o;
/*  765 */       return (equalCachedHash(that)) && (this.elementType.equals(that.elementType)) && (this.props.equals(that.props));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  770 */     int computeHash() { return super.computeHash() + this.elementType.computeHash(); }
/*      */     
/*      */     void toJson(Schema.Names names, JsonGenerator gen) throws IOException {
/*  773 */       gen.writeStartObject();
/*  774 */       gen.writeStringField("type", "array");
/*  775 */       gen.writeFieldName("items");
/*  776 */       this.elementType.toJson(names, gen);
/*  777 */       this.props.write(gen);
/*  778 */       gen.writeEndObject();
/*      */     }
/*      */   }
/*      */   
/*      */   private static class MapSchema extends Schema {
/*      */     private final Schema valueType;
/*      */     
/*  785 */     public MapSchema(Schema valueType) { super();
/*  786 */       this.valueType = valueType; }
/*      */     
/*  788 */     public Schema getValueType() { return this.valueType; }
/*      */     
/*  790 */     public boolean equals(Object o) { if (o == this) return true;
/*  791 */       if (!(o instanceof MapSchema)) return false;
/*  792 */       MapSchema that = (MapSchema)o;
/*  793 */       return (equalCachedHash(that)) && (this.valueType.equals(that.valueType)) && (this.props.equals(that.props));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  798 */     int computeHash() { return super.computeHash() + this.valueType.computeHash(); }
/*      */     
/*      */     void toJson(Schema.Names names, JsonGenerator gen) throws IOException {
/*  801 */       gen.writeStartObject();
/*  802 */       gen.writeStringField("type", "map");
/*  803 */       gen.writeFieldName("values");
/*  804 */       this.valueType.toJson(names, gen);
/*  805 */       this.props.write(gen);
/*  806 */       gen.writeEndObject();
/*      */     }
/*      */   }
/*      */   
/*      */   private static class UnionSchema extends Schema {
/*      */     private final List<Schema> types;
/*  812 */     private final Map<String, Integer> indexByName = new HashMap();
/*      */     
/*      */     public UnionSchema(Schema.LockableArrayList<Schema> types) {
/*  815 */       super();
/*  816 */       this.types = types.lock();
/*  817 */       int index = 0;
/*  818 */       for (Schema type : types) {
/*  819 */         if (type.getType() == Schema.Type.UNION)
/*  820 */           throw new AvroRuntimeException("Nested union: " + this);
/*  821 */         String name = type.getFullName();
/*  822 */         if (name == null)
/*  823 */           throw new AvroRuntimeException("Nameless in union:" + this);
/*  824 */         if (this.indexByName.put(name, Integer.valueOf(index++)) != null)
/*  825 */           throw new AvroRuntimeException("Duplicate in union:" + name);
/*      */       } }
/*      */     
/*  828 */     public List<Schema> getTypes() { return this.types; }
/*  829 */     public Integer getIndexNamed(String name) { return (Integer)this.indexByName.get(name); }
/*      */     
/*  831 */     public boolean equals(Object o) { if (o == this) return true;
/*  832 */       if (!(o instanceof UnionSchema)) return false;
/*  833 */       UnionSchema that = (UnionSchema)o;
/*  834 */       return (equalCachedHash(that)) && (this.types.equals(that.types)) && (this.props.equals(that.props));
/*      */     }
/*      */     
/*      */     int computeHash()
/*      */     {
/*  839 */       int hash = super.computeHash();
/*  840 */       for (Schema type : this.types)
/*  841 */         hash += type.computeHash();
/*  842 */       return hash;
/*      */     }
/*      */     
/*      */     public void addProp(String name, String value)
/*      */     {
/*  847 */       throw new AvroRuntimeException("Can't set properties on a union: " + this);
/*      */     }
/*      */     
/*      */     void toJson(Schema.Names names, JsonGenerator gen) throws IOException {
/*  851 */       gen.writeStartArray();
/*  852 */       for (Schema type : this.types)
/*  853 */         type.toJson(names, gen);
/*  854 */       gen.writeEndArray();
/*      */     }
/*      */   }
/*      */   
/*      */   private static class FixedSchema extends Schema.NamedSchema {
/*      */     private final int size;
/*      */     
/*  861 */     public FixedSchema(Schema.Name name, String doc, int size) { super(name, doc);
/*  862 */       if (size < 0)
/*  863 */         throw new IllegalArgumentException("Invalid fixed size: " + size);
/*  864 */       this.size = size; }
/*      */     
/*  866 */     public int getFixedSize() { return this.size; }
/*      */     
/*  868 */     public boolean equals(Object o) { if (o == this) return true;
/*  869 */       if (!(o instanceof FixedSchema)) return false;
/*  870 */       FixedSchema that = (FixedSchema)o;
/*  871 */       return (equalCachedHash(that)) && (equalNames(that)) && (this.size == that.size) && (this.props.equals(that.props));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  876 */     int computeHash() { return super.computeHash() + this.size; }
/*      */     
/*  878 */     void toJson(Schema.Names names, JsonGenerator gen) throws IOException { if (writeNameRef(names, gen)) return;
/*  879 */       gen.writeStartObject();
/*  880 */       gen.writeStringField("type", "fixed");
/*  881 */       writeName(names, gen);
/*  882 */       if (getDoc() != null)
/*  883 */         gen.writeStringField("doc", getDoc());
/*  884 */       gen.writeNumberField("size", this.size);
/*  885 */       this.props.write(gen);
/*  886 */       aliasesToJson(gen);
/*  887 */       gen.writeEndObject();
/*      */     }
/*      */   }
/*      */   
/*      */   private static class StringSchema extends Schema {
/*  892 */     public StringSchema() { super(); }
/*      */   }
/*      */   
/*      */   private static class BytesSchema extends Schema {
/*  896 */     public BytesSchema() { super(); }
/*      */   }
/*      */   
/*      */   private static class IntSchema extends Schema {
/*  900 */     public IntSchema() { super(); }
/*      */   }
/*      */   
/*      */   private static class LongSchema extends Schema {
/*  904 */     public LongSchema() { super(); }
/*      */   }
/*      */   
/*      */   private static class FloatSchema extends Schema {
/*  908 */     public FloatSchema() { super(); }
/*      */   }
/*      */   
/*      */   private static class DoubleSchema extends Schema {
/*  912 */     public DoubleSchema() { super(); }
/*      */   }
/*      */   
/*      */   private static class BooleanSchema extends Schema {
/*  916 */     public BooleanSchema() { super(); }
/*      */   }
/*      */   
/*      */   private static class NullSchema extends Schema {
/*  920 */     public NullSchema() { super(); }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public static class Parser
/*      */   {
/*  927 */     private Schema.Names names = new Schema.Names();
/*  928 */     private boolean validate = true;
/*      */     
/*      */ 
/*      */     public Parser addTypes(Map<String, Schema> types)
/*      */     {
/*  933 */       for (Schema s : types.values())
/*  934 */         this.names.add(s);
/*  935 */       return this;
/*      */     }
/*      */     
/*      */     public Map<String, Schema> getTypes()
/*      */     {
/*  940 */       Map<String, Schema> result = new LinkedHashMap();
/*  941 */       for (Schema s : this.names.values())
/*  942 */         result.put(s.getFullName(), s);
/*  943 */       return result;
/*      */     }
/*      */     
/*      */     public Parser setValidate(boolean validate)
/*      */     {
/*  948 */       this.validate = validate;
/*  949 */       return this;
/*      */     }
/*      */     
/*      */     public boolean getValidate() {
/*  953 */       return this.validate;
/*      */     }
/*      */     
/*      */     public Schema parse(File file) throws IOException
/*      */     {
/*  958 */       return parse(Schema.FACTORY.createJsonParser(file));
/*      */     }
/*      */     
/*      */     public Schema parse(InputStream in)
/*      */       throws IOException
/*      */     {
/*  964 */       return parse(Schema.FACTORY.createJsonParser(in));
/*      */     }
/*      */     
/*      */     public Schema parse(String s)
/*      */     {
/*      */       try
/*      */       {
/*  971 */         return parse(Schema.FACTORY.createJsonParser(new StringReader(s)));
/*      */       } catch (IOException e) {
/*  973 */         throw new SchemaParseException(e);
/*      */       }
/*      */     }
/*      */     
/*      */     private Schema parse(JsonParser parser) throws IOException {
/*  978 */       boolean saved = ((Boolean)Schema.validateNames.get()).booleanValue();
/*      */       try {
/*  980 */         Schema.validateNames.set(Boolean.valueOf(this.validate));
/*  981 */         return Schema.parse(Schema.MAPPER.readTree(parser), this.names);
/*      */       } catch (JsonParseException e) {
/*  983 */         throw new SchemaParseException(e);
/*      */       } finally {
/*  985 */         Schema.validateNames.set(Boolean.valueOf(saved));
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   /**
/*      */    * @deprecated
/*      */    */
/*      */   public static Schema parse(File file)
/*      */     throws IOException
/*      */   {
/* 1000 */     return new Parser().parse(file);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   /**
/*      */    * @deprecated
/*      */    */
/*      */   public static Schema parse(InputStream in)
/*      */     throws IOException
/*      */   {
/* 1013 */     return new Parser().parse(in);
/*      */   }
/*      */   
/*      */   /**
/*      */    * @deprecated
/*      */    */
/*      */   public static Schema parse(String jsonSchema) {
/* 1020 */     return new Parser().parse(jsonSchema);
/*      */   }
/*      */   
/*      */   /**
/*      */    * @deprecated
/*      */    */
/*      */   public static Schema parse(String jsonSchema, boolean validate)
/*      */   {
/* 1028 */     return new Parser().setValidate(validate).parse(jsonSchema);
/*      */   }
/*      */   
/*      */   static
/*      */   {
/*   85 */     FACTORY.enable(JsonParser.Feature.ALLOW_COMMENTS);
/*   86 */     FACTORY.setCodec(MAPPER);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  143 */     SCHEMA_RESERVED = new HashSet();
/*      */     
/*  145 */     Collections.addAll(SCHEMA_RESERVED, new String[] { "doc", "fields", "items", "name", "namespace", "size", "symbols", "values", "type" });
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  381 */     FIELD_RESERVED = new HashSet();
/*      */     
/*  383 */     Collections.addAll(FIELD_RESERVED, new String[] { "default", "doc", "name", "order", "type" });
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  585 */     SEEN_EQUALS = new ThreadLocal() {
/*  586 */       protected Set initialValue() { return new HashSet(); }
/*  587 */     };
/*  588 */     SEEN_HASHCODE = new ThreadLocal() {
/*  589 */       protected Map initialValue() { return new IdentityHashMap();
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1030 */     };
/* 1031 */     PRIMITIVES = new HashMap();
/*      */     
/* 1033 */     PRIMITIVES.put("string", Type.STRING);
/* 1034 */     PRIMITIVES.put("bytes", Type.BYTES);
/* 1035 */     PRIMITIVES.put("int", Type.INT);
/* 1036 */     PRIMITIVES.put("long", Type.LONG);
/* 1037 */     PRIMITIVES.put("float", Type.FLOAT);
/* 1038 */     PRIMITIVES.put("double", Type.DOUBLE);
/* 1039 */     PRIMITIVES.put("boolean", Type.BOOLEAN);
/* 1040 */     PRIMITIVES.put("null", Type.NULL);
/*      */   }
/*      */   
/*      */   static class Names extends LinkedHashMap<Schema.Name, Schema> { private String space;
/*      */     
/*      */     public Names() {}
/*      */     
/* 1047 */     public Names(String space) { this.space = space; }
/*      */     
/* 1049 */     public String space() { return this.space; }
/* 1050 */     public void space(String space) { this.space = space; }
/*      */     
/*      */     public Schema get(Object o) {
/*      */       Schema.Name name;
/*      */       Schema.Name name;
/* 1055 */       if ((o instanceof String)) {
/* 1056 */         Schema.Type primitive = (Schema.Type)Schema.PRIMITIVES.get((String)o);
/* 1057 */         if (primitive != null) return Schema.create(primitive);
/* 1058 */         name = new Schema.Name((String)o, this.space);
/*      */       } else {
/* 1060 */         name = (Schema.Name)o;
/*      */       }
/* 1062 */       return (Schema)super.get(name);
/*      */     }
/*      */     
/* 1065 */     public boolean contains(Schema schema) { return get(((Schema.NamedSchema)schema).name) != null; }
/*      */     
/*      */     public void add(Schema schema) {
/* 1068 */       put(((Schema.NamedSchema)schema).name, schema);
/*      */     }
/*      */     
/*      */     public Schema put(Schema.Name name, Schema schema) {
/* 1072 */       if (containsKey(name))
/* 1073 */         throw new SchemaParseException("Can't redefine: " + name);
/* 1074 */       return (Schema)super.put(name, schema);
/*      */     }
/*      */   }
/*      */   
/* 1078 */   private static ThreadLocal<Boolean> validateNames = new ThreadLocal()
/*      */   {
/*      */     protected Boolean initialValue() {
/* 1081 */       return Boolean.valueOf(true);
/*      */     }
/*      */   };
/*      */   
/*      */   private static String validateName(String name) {
/* 1086 */     if (!((Boolean)validateNames.get()).booleanValue()) return name;
/* 1087 */     int length = name.length();
/* 1088 */     if (length == 0)
/* 1089 */       throw new SchemaParseException("Empty name");
/* 1090 */     char first = name.charAt(0);
/* 1091 */     if ((!Character.isLetter(first)) && (first != '_'))
/* 1092 */       throw new SchemaParseException("Illegal initial character: " + name);
/* 1093 */     for (int i = 1; i < length; i++) {
/* 1094 */       char c = name.charAt(i);
/* 1095 */       if ((!Character.isLetterOrDigit(c)) && (c != '_'))
/* 1096 */         throw new SchemaParseException("Illegal character in: " + name);
/*      */     }
/* 1098 */     return name;
/*      */   }
/*      */   
/*      */   static Schema parse(JsonNode schema, Names names)
/*      */   {
/* 1103 */     if (schema.isTextual()) {
/* 1104 */       Schema result = names.get(schema.getTextValue());
/* 1105 */       if (result == null)
/* 1106 */         throw new SchemaParseException("Undefined name: " + schema);
/* 1107 */       return result; }
/* 1108 */     if (schema.isObject())
/*      */     {
/* 1110 */       String type = getRequiredText(schema, "type", "No type");
/* 1111 */       Name name = null;
/* 1112 */       String savedSpace = null;
/* 1113 */       String doc = null;
/* 1114 */       if ((type.equals("record")) || (type.equals("error")) || (type.equals("enum")) || (type.equals("fixed")))
/*      */       {
/* 1116 */         String space = getOptionalText(schema, "namespace");
/* 1117 */         doc = getOptionalText(schema, "doc");
/* 1118 */         if (space == null)
/* 1119 */           space = names.space();
/* 1120 */         name = new Name(getRequiredText(schema, "name", "No name in schema"), space);
/*      */         
/* 1122 */         if (name.space != null) {
/* 1123 */           savedSpace = names.space();
/* 1124 */           names.space(name.space);
/*      */         } }
/*      */       Schema result;
/* 1127 */       if (PRIMITIVES.containsKey(type)) {
/* 1128 */         result = create((Type)PRIMITIVES.get(type));
/* 1129 */       } else if ((type.equals("record")) || (type.equals("error"))) {
/* 1130 */         List<Field> fields = new ArrayList();
/* 1131 */         Schema result = new RecordSchema(name, doc, type.equals("error"));
/* 1132 */         if (name != null) names.add(result);
/* 1133 */         JsonNode fieldsNode = schema.get("fields");
/* 1134 */         if ((fieldsNode == null) || (!fieldsNode.isArray()))
/* 1135 */           throw new SchemaParseException("Record has no fields: " + schema);
/* 1136 */         for (JsonNode field : fieldsNode) {
/* 1137 */           String fieldName = getRequiredText(field, "name", "No field name");
/* 1138 */           String fieldDoc = getOptionalText(field, "doc");
/* 1139 */           JsonNode fieldTypeNode = field.get("type");
/* 1140 */           if (fieldTypeNode == null)
/* 1141 */             throw new SchemaParseException("No field type: " + field);
/* 1142 */           if ((fieldTypeNode.isTextual()) && (names.get(fieldTypeNode.getTextValue()) == null))
/*      */           {
/* 1144 */             throw new SchemaParseException(fieldTypeNode + " is not a defined name." + " The type of the \"" + fieldName + "\" field must be" + " a defined name or a {\"type\": ...} expression.");
/*      */           }
/*      */           
/*      */ 
/* 1148 */           Schema fieldSchema = parse(fieldTypeNode, names);
/* 1149 */           Schema.Field.Order order = Schema.Field.Order.ASCENDING;
/* 1150 */           JsonNode orderNode = field.get("order");
/* 1151 */           if (orderNode != null)
/* 1152 */             order = Schema.Field.Order.valueOf(orderNode.getTextValue().toUpperCase());
/* 1153 */           JsonNode defaultValue = field.get("default");
/* 1154 */           if ((defaultValue != null) && ((Type.FLOAT.equals(fieldSchema.getType())) || (Type.DOUBLE.equals(fieldSchema.getType()))) && (defaultValue.isTextual()))
/*      */           {
/*      */ 
/*      */ 
/* 1158 */             defaultValue = new DoubleNode(Double.valueOf(defaultValue.getTextValue()).doubleValue());
/*      */           }
/* 1160 */           Field f = new Field(fieldName, fieldSchema, fieldDoc, defaultValue, order);
/*      */           
/* 1162 */           Iterator<String> i = field.getFieldNames();
/* 1163 */           while (i.hasNext()) {
/* 1164 */             String prop = (String)i.next();
/* 1165 */             String value = field.get(prop).getTextValue();
/* 1166 */             if ((!FIELD_RESERVED.contains(prop)) && (value != null))
/* 1167 */               f.addProp(prop, value);
/*      */           }
/* 1169 */           f.aliases = parseAliases(field);
/* 1170 */           fields.add(f);
/*      */         }
/* 1172 */         result.setFields(fields);
/* 1173 */       } else if (type.equals("enum")) {
/* 1174 */         JsonNode symbolsNode = schema.get("symbols");
/* 1175 */         if ((symbolsNode == null) || (!symbolsNode.isArray()))
/* 1176 */           throw new SchemaParseException("Enum has no symbols: " + schema);
/* 1177 */         LockableArrayList<String> symbols = new LockableArrayList();
/* 1178 */         for (JsonNode n : symbolsNode)
/* 1179 */           symbols.add(n.getTextValue());
/* 1180 */         Schema result = new EnumSchema(name, doc, symbols);
/* 1181 */         if (name != null) names.add(result); } else { Schema result;
/* 1182 */         if (type.equals("array")) {
/* 1183 */           JsonNode itemsNode = schema.get("items");
/* 1184 */           if (itemsNode == null)
/* 1185 */             throw new SchemaParseException("Array has no items type: " + schema);
/* 1186 */           result = new ArraySchema(parse(itemsNode, names)); } else { Schema result;
/* 1187 */           if (type.equals("map")) {
/* 1188 */             JsonNode valuesNode = schema.get("values");
/* 1189 */             if (valuesNode == null)
/* 1190 */               throw new SchemaParseException("Map has no values type: " + schema);
/* 1191 */             result = new MapSchema(parse(valuesNode, names));
/* 1192 */           } else if (type.equals("fixed")) {
/* 1193 */             JsonNode sizeNode = schema.get("size");
/* 1194 */             if ((sizeNode == null) || (!sizeNode.isInt()))
/* 1195 */               throw new SchemaParseException("Invalid or no size: " + schema);
/* 1196 */             Schema result = new FixedSchema(name, doc, sizeNode.getIntValue());
/* 1197 */             if (name != null) names.add(result);
/*      */           } else {
/* 1199 */             throw new SchemaParseException("Type not supported: " + type); } } }
/* 1200 */       Schema result; Iterator<String> i = schema.getFieldNames();
/* 1201 */       while (i.hasNext()) {
/* 1202 */         String prop = (String)i.next();
/* 1203 */         String value = schema.get(prop).getTextValue();
/* 1204 */         if ((!SCHEMA_RESERVED.contains(prop)) && (value != null))
/* 1205 */           result.addProp(prop, value);
/*      */       }
/* 1207 */       if (savedSpace != null)
/* 1208 */         names.space(savedSpace);
/* 1209 */       if ((result instanceof NamedSchema)) {
/* 1210 */         Set<String> aliases = parseAliases(schema);
/* 1211 */         if (aliases != null)
/* 1212 */           for (String alias : aliases)
/* 1213 */             result.addAlias(alias);
/*      */       }
/* 1215 */       return result; }
/* 1216 */     if (schema.isArray()) {
/* 1217 */       LockableArrayList<Schema> types = new LockableArrayList(schema.size());
/*      */       
/* 1219 */       for (JsonNode typeNode : schema)
/* 1220 */         types.add(parse(typeNode, names));
/* 1221 */       return new UnionSchema(types);
/*      */     }
/* 1223 */     throw new SchemaParseException("Schema not yet supported: " + schema);
/*      */   }
/*      */   
/*      */   private static Set<String> parseAliases(JsonNode node)
/*      */   {
/* 1228 */     JsonNode aliasesNode = node.get("aliases");
/* 1229 */     if (aliasesNode == null)
/* 1230 */       return null;
/* 1231 */     if (!aliasesNode.isArray())
/* 1232 */       throw new SchemaParseException("aliases not an array: " + node);
/* 1233 */     Set<String> aliases = new LinkedHashSet();
/* 1234 */     for (JsonNode aliasNode : aliasesNode) {
/* 1235 */       if (!aliasNode.isTextual())
/* 1236 */         throw new SchemaParseException("alias not a string: " + aliasNode);
/* 1237 */       aliases.add(aliasNode.getTextValue());
/*      */     }
/* 1239 */     return aliases;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static String getRequiredText(JsonNode container, String key, String error)
/*      */   {
/* 1252 */     String out = getOptionalText(container, key);
/* 1253 */     if (null == out) {
/* 1254 */       throw new SchemaParseException(error + ": " + container);
/*      */     }
/* 1256 */     return out;
/*      */   }
/*      */   
/*      */   private static String getOptionalText(JsonNode container, String key)
/*      */   {
/* 1261 */     JsonNode jsonNode = container.get(key);
/* 1262 */     return jsonNode != null ? jsonNode.getTextValue() : null;
/*      */   }
/*      */   
/*      */   static JsonNode parseJson(String s) {
/*      */     try {
/* 1267 */       return MAPPER.readTree(FACTORY.createJsonParser(new StringReader(s)));
/*      */     } catch (JsonParseException e) {
/* 1269 */       throw new RuntimeException(e);
/*      */     } catch (IOException e) {
/* 1271 */       throw new RuntimeException(e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static Schema applyAliases(Schema writer, Schema reader)
/*      */   {
/* 1281 */     if (writer == reader) { return writer;
/*      */     }
/*      */     
/* 1284 */     Map<Schema, Schema> seen = new IdentityHashMap(1);
/* 1285 */     Map<Name, Name> aliases = new HashMap(1);
/* 1286 */     Map<Name, Map<String, String>> fieldAliases = new HashMap(1);
/*      */     
/* 1288 */     getAliases(reader, seen, aliases, fieldAliases);
/*      */     
/* 1290 */     if ((aliases.size() == 0) && (fieldAliases.size() == 0)) {
/* 1291 */       return writer;
/*      */     }
/* 1293 */     seen.clear();
/* 1294 */     return applyAliases(writer, seen, aliases, fieldAliases);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private static Schema applyAliases(Schema s, Map<Schema, Schema> seen, Map<Name, Name> aliases, Map<Name, Map<String, String>> fieldAliases)
/*      */   {
/* 1301 */     Name name = (s instanceof NamedSchema) ? ((NamedSchema)s).name : null;
/* 1302 */     Schema result = s;
/* 1303 */     switch (s.getType()) {
/*      */     case RECORD: 
/* 1305 */       if (seen.containsKey(s)) return (Schema)seen.get(s);
/* 1306 */       if (aliases.containsKey(name))
/* 1307 */         name = (Name)aliases.get(name);
/* 1308 */       result = createRecord(name.full, s.getDoc(), null, s.isError());
/* 1309 */       seen.put(s, result);
/* 1310 */       List<Field> newFields = new ArrayList();
/* 1311 */       for (Field f : s.getFields()) {
/* 1312 */         Schema fSchema = applyAliases(f.schema, seen, aliases, fieldAliases);
/* 1313 */         String fName = getFieldAlias(name, f.name, fieldAliases);
/* 1314 */         Field newF = new Field(fName, fSchema, f.doc, f.defaultValue, f.order);
/* 1315 */         newF.props.putAll(f.props);
/* 1316 */         newFields.add(newF);
/*      */       }
/* 1318 */       result.setFields(newFields);
/* 1319 */       break;
/*      */     case ENUM: 
/* 1321 */       if (aliases.containsKey(name)) {
/* 1322 */         result = createEnum(((Name)aliases.get(name)).full, s.getDoc(), null, s.getEnumSymbols());
/*      */       }
/*      */       break;
/*      */     case ARRAY: 
/* 1326 */       Schema e = applyAliases(s.getElementType(), seen, aliases, fieldAliases);
/* 1327 */       if (e != s.getElementType())
/* 1328 */         result = createArray(e);
/*      */       break;
/*      */     case MAP: 
/* 1331 */       Schema v = applyAliases(s.getValueType(), seen, aliases, fieldAliases);
/* 1332 */       if (v != s.getValueType())
/* 1333 */         result = createMap(v);
/*      */       break;
/*      */     case UNION: 
/* 1336 */       List<Schema> types = new ArrayList();
/* 1337 */       for (Schema branch : s.getTypes())
/* 1338 */         types.add(applyAliases(branch, seen, aliases, fieldAliases));
/* 1339 */       result = createUnion(types);
/* 1340 */       break;
/*      */     case FIXED: 
/* 1342 */       if (aliases.containsKey(name)) {
/* 1343 */         result = createFixed(((Name)aliases.get(name)).full, s.getDoc(), null, s.getFixedSize());
/*      */       }
/*      */       break;
/*      */     }
/* 1347 */     if (result != s)
/* 1348 */       result.props.putAll(s.props);
/* 1349 */     return result;
/*      */   }
/*      */   
/*      */ 
/*      */   private static void getAliases(Schema schema, Map<Schema, Schema> seen, Map<Name, Name> aliases, Map<Name, Map<String, String>> fieldAliases)
/*      */   {
/*      */     NamedSchema namedSchema;
/*      */     
/* 1357 */     if ((schema instanceof NamedSchema)) {
/* 1358 */       namedSchema = (NamedSchema)schema;
/* 1359 */       if (namedSchema.aliases != null)
/* 1360 */         for (Name alias : namedSchema.aliases)
/* 1361 */           aliases.put(alias, namedSchema.name); }
/*      */     RecordSchema record;
/* 1363 */     switch (schema.getType()) {
/*      */     case RECORD: 
/* 1365 */       if (seen.containsKey(schema)) return;
/* 1366 */       seen.put(schema, schema);
/* 1367 */       record = (RecordSchema)schema;
/* 1368 */       for (Field field : schema.getFields()) {
/* 1369 */         if (field.aliases != null)
/* 1370 */           for (String fieldAlias : field.aliases) {
/* 1371 */             Map<String, String> recordAliases = (Map)fieldAliases.get(record.name);
/* 1372 */             if (recordAliases == null) {
/* 1373 */               fieldAliases.put(record.name, recordAliases = new HashMap());
/*      */             }
/* 1375 */             recordAliases.put(fieldAlias, field.name);
/*      */           }
/* 1377 */         getAliases(field.schema, seen, aliases, fieldAliases);
/*      */       }
/* 1379 */       if ((record.aliases != null) && (fieldAliases.containsKey(record.name)))
/* 1380 */         for (Name recordAlias : record.aliases)
/* 1381 */           fieldAliases.put(recordAlias, fieldAliases.get(record.name)); break;
/*      */     
/*      */     case ARRAY: 
/* 1384 */       getAliases(schema.getElementType(), seen, aliases, fieldAliases);
/* 1385 */       break;
/*      */     case MAP: 
/* 1387 */       getAliases(schema.getValueType(), seen, aliases, fieldAliases);
/* 1388 */       break;
/*      */     case UNION: 
/* 1390 */       for (Schema s : schema.getTypes()) {
/* 1391 */         getAliases(s, seen, aliases, fieldAliases);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private static String getFieldAlias(Name record, String field, Map<Name, Map<String, String>> fieldAliases)
/*      */   {
/* 1398 */     Map<String, String> recordAliases = (Map)fieldAliases.get(record);
/* 1399 */     if (recordAliases == null)
/* 1400 */       return field;
/* 1401 */     String alias = (String)recordAliases.get(field);
/* 1402 */     if (alias == null)
/* 1403 */       return field;
/* 1404 */     return alias;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   static class LockableArrayList<E>
/*      */     extends ArrayList<E>
/*      */   {
/*      */     private static final long serialVersionUID = 1L;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1426 */     private boolean locked = false;
/*      */     
/*      */     public LockableArrayList() {}
/*      */     
/*      */     public LockableArrayList(int size)
/*      */     {
/* 1432 */       super();
/*      */     }
/*      */     
/*      */     public LockableArrayList(List<E> types) {
/* 1436 */       super();
/*      */     }
/*      */     
/*      */     public List<E> lock() {
/* 1440 */       this.locked = true;
/* 1441 */       return this;
/*      */     }
/*      */     
/*      */     private void ensureUnlocked() {
/* 1445 */       if (this.locked) {
/* 1446 */         throw new IllegalStateException();
/*      */       }
/*      */     }
/*      */     
/*      */     public boolean add(E e) {
/* 1451 */       ensureUnlocked();
/* 1452 */       return super.add(e);
/*      */     }
/*      */     
/*      */     public boolean remove(Object o) {
/* 1456 */       ensureUnlocked();
/* 1457 */       return super.remove(o);
/*      */     }
/*      */     
/*      */     public E remove(int index) {
/* 1461 */       ensureUnlocked();
/* 1462 */       return (E)super.remove(index);
/*      */     }
/*      */     
/*      */     public boolean addAll(Collection<? extends E> c) {
/* 1466 */       ensureUnlocked();
/* 1467 */       return super.addAll(c);
/*      */     }
/*      */     
/*      */     public boolean addAll(int index, Collection<? extends E> c) {
/* 1471 */       ensureUnlocked();
/* 1472 */       return super.addAll(index, c);
/*      */     }
/*      */     
/*      */     public boolean removeAll(Collection<?> c) {
/* 1476 */       ensureUnlocked();
/* 1477 */       return super.removeAll(c);
/*      */     }
/*      */     
/*      */     public boolean retainAll(Collection<?> c) {
/* 1481 */       ensureUnlocked();
/* 1482 */       return super.retainAll(c);
/*      */     }
/*      */     
/*      */     public void clear() {
/* 1486 */       ensureUnlocked();
/* 1487 */       super.clear();
/*      */     }
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\Schema.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */