/*     */ package org.codehaus.jackson.map.jsontype.impl;
/*     */ 
/*     */ import java.util.Collection;
/*     */ import org.codehaus.jackson.annotate.JsonTypeInfo.As;
/*     */ import org.codehaus.jackson.annotate.JsonTypeInfo.Id;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.DeserializationConfig;
/*     */ import org.codehaus.jackson.map.MapperConfig;
/*     */ import org.codehaus.jackson.map.SerializationConfig;
/*     */ import org.codehaus.jackson.map.TypeDeserializer;
/*     */ import org.codehaus.jackson.map.TypeSerializer;
/*     */ import org.codehaus.jackson.map.jsontype.NamedType;
/*     */ import org.codehaus.jackson.map.jsontype.TypeIdResolver;
/*     */ import org.codehaus.jackson.map.jsontype.TypeResolverBuilder;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class StdTypeResolverBuilder
/*     */   implements TypeResolverBuilder<StdTypeResolverBuilder>
/*     */ {
/*     */   protected JsonTypeInfo.Id _idType;
/*     */   protected JsonTypeInfo.As _includeAs;
/*     */   protected String _typeProperty;
/*     */   protected TypeIdResolver _customIdResolver;
/*     */   
/*     */   public StdTypeResolverBuilder init(JsonTypeInfo.Id idType, TypeIdResolver idRes)
/*     */   {
/*  44 */     if (idType == null) {
/*  45 */       throw new IllegalArgumentException("idType can not be null");
/*     */     }
/*  47 */     this._idType = idType;
/*  48 */     this._customIdResolver = idRes;
/*     */     
/*  50 */     this._typeProperty = idType.getDefaultPropertyName();
/*  51 */     return this;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public TypeSerializer buildTypeSerializer(SerializationConfig config, JavaType baseType, Collection<NamedType> subtypes, BeanProperty property)
/*     */   {
/*  58 */     TypeIdResolver idRes = idResolver(config, baseType, subtypes, true, false);
/*  59 */     switch (this._includeAs) {
/*     */     case WRAPPER_ARRAY: 
/*  61 */       return new AsArrayTypeSerializer(idRes, property);
/*     */     case PROPERTY: 
/*  63 */       return new AsPropertyTypeSerializer(idRes, property, this._typeProperty);
/*     */     case WRAPPER_OBJECT: 
/*  65 */       return new AsWrapperTypeSerializer(idRes, property);
/*     */     }
/*  67 */     throw new IllegalStateException("Do not know how to construct standard type serializer for inclusion type: " + this._includeAs);
/*     */   }
/*     */   
/*     */ 
/*     */   public TypeDeserializer buildTypeDeserializer(DeserializationConfig config, JavaType baseType, Collection<NamedType> subtypes, BeanProperty property)
/*     */   {
/*  73 */     TypeIdResolver idRes = idResolver(config, baseType, subtypes, false, true);
/*     */     
/*     */ 
/*  76 */     switch (this._includeAs) {
/*     */     case WRAPPER_ARRAY: 
/*  78 */       return new AsArrayTypeDeserializer(baseType, idRes, property);
/*     */     case PROPERTY: 
/*  80 */       return new AsPropertyTypeDeserializer(baseType, idRes, property, this._typeProperty);
/*     */     case WRAPPER_OBJECT: 
/*  82 */       return new AsWrapperTypeDeserializer(baseType, idRes, property);
/*     */     }
/*  84 */     throw new IllegalStateException("Do not know how to construct standard type serializer for inclusion type: " + this._includeAs);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public StdTypeResolverBuilder inclusion(JsonTypeInfo.As includeAs)
/*     */   {
/*  94 */     if (includeAs == null) {
/*  95 */       throw new IllegalArgumentException("includeAs can not be null");
/*     */     }
/*  97 */     this._includeAs = includeAs;
/*  98 */     return this;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public StdTypeResolverBuilder typeProperty(String typeIdPropName)
/*     */   {
/* 108 */     if ((typeIdPropName == null) || (typeIdPropName.length() == 0)) {
/* 109 */       typeIdPropName = this._idType.getDefaultPropertyName();
/*     */     }
/* 111 */     this._typeProperty = typeIdPropName;
/* 112 */     return this;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public String getTypeProperty()
/*     */   {
/* 121 */     return this._typeProperty;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected TypeIdResolver idResolver(MapperConfig<?> config, JavaType baseType, Collection<NamedType> subtypes, boolean forSer, boolean forDeser)
/*     */   {
/* 139 */     if (this._customIdResolver != null) {
/* 140 */       return this._customIdResolver;
/*     */     }
/* 142 */     if (this._idType == null) {
/* 143 */       throw new IllegalStateException("Can not build, 'init()' not yet called");
/*     */     }
/* 145 */     switch (this._idType) {
/*     */     case CLASS: 
/* 147 */       return new ClassNameIdResolver(baseType, config.getTypeFactory());
/*     */     case MINIMAL_CLASS: 
/* 149 */       return new MinimalClassNameIdResolver(baseType, config.getTypeFactory());
/*     */     case NAME: 
/* 151 */       return TypeNameIdResolver.construct(config, baseType, subtypes, forSer, forDeser);
/*     */     }
/*     */     
/*     */     
/*     */ 
/* 156 */     throw new IllegalStateException("Do not know how to construct standard type id resolver for idType: " + this._idType);
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\jsontype\impl\StdTypeResolverBuilder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */