/*     */ package org.codehaus.jackson.map.ser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.Type;
/*     */ import java.math.BigDecimal;
/*     */ import java.math.BigInteger;
/*     */ import java.sql.Time;
/*     */ import java.util.Calendar;
/*     */ import org.codehaus.jackson.JsonGenerationException;
/*     */ import org.codehaus.jackson.JsonGenerator;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.JsonSerializable;
/*     */ import org.codehaus.jackson.map.JsonSerializableWithType;
/*     */ import org.codehaus.jackson.map.ObjectMapper;
/*     */ import org.codehaus.jackson.map.SerializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.SerializerProvider;
/*     */ import org.codehaus.jackson.map.TypeSerializer;
/*     */ import org.codehaus.jackson.map.annotate.JacksonStdImpl;
/*     */ import org.codehaus.jackson.map.type.TypeFactory;
/*     */ import org.codehaus.jackson.node.ObjectNode;
/*     */ import org.codehaus.jackson.schema.JsonSerializableSchema;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ import org.codehaus.jackson.util.TokenBuffer;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class StdSerializers
/*     */ {
/*     */   protected static abstract class NonTypedScalarSerializer<T>
/*     */     extends ScalarSerializerBase<T>
/*     */   {
/*     */     protected NonTypedScalarSerializer(Class<T> t)
/*     */     {
/*  45 */       super();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public final void serializeWithType(T value, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/*  54 */       serialize(value, jgen, provider);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class BooleanSerializer
/*     */     extends StdSerializers.NonTypedScalarSerializer<Boolean>
/*     */   {
/*     */     final boolean _forPrimitive;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public BooleanSerializer(boolean forPrimitive)
/*     */     {
/*  83 */       super();
/*  84 */       this._forPrimitive = forPrimitive;
/*     */     }
/*     */     
/*     */ 
/*     */     public void serialize(Boolean value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/*  91 */       jgen.writeBoolean(value.booleanValue());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 103 */       return createSchemaNode("boolean", !this._forPrimitive);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class StringSerializer
/*     */     extends StdSerializers.NonTypedScalarSerializer<String>
/*     */   {
/*     */     public StringSerializer()
/*     */     {
/* 117 */       super();
/*     */     }
/*     */     
/*     */     public void serialize(String value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 123 */       jgen.writeString(value);
/*     */     }
/*     */     
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 129 */       return createSchemaNode("string", true);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class IntegerSerializer
/*     */     extends StdSerializers.NonTypedScalarSerializer<Integer>
/*     */   {
/*     */     public IntegerSerializer()
/*     */     {
/* 150 */       super();
/*     */     }
/*     */     
/*     */     public void serialize(Integer value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 156 */       jgen.writeNumber(value.intValue());
/*     */     }
/*     */     
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 162 */       return createSchemaNode("integer", true);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class IntLikeSerializer
/*     */     extends ScalarSerializerBase<Number>
/*     */   {
/* 175 */     static final IntLikeSerializer instance = new IntLikeSerializer();
/*     */     
/* 177 */     public IntLikeSerializer() { super(); }
/*     */     
/*     */ 
/*     */     public void serialize(Number value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 183 */       jgen.writeNumber(value.intValue());
/*     */     }
/*     */     
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 189 */       return createSchemaNode("integer", true);
/*     */     }
/*     */   }
/*     */   
/*     */   @JacksonStdImpl
/*     */   public static final class LongSerializer
/*     */     extends ScalarSerializerBase<Long>
/*     */   {
/* 197 */     static final LongSerializer instance = new LongSerializer();
/*     */     
/* 199 */     public LongSerializer() { super(); }
/*     */     
/*     */ 
/*     */     public void serialize(Long value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 205 */       jgen.writeNumber(value.longValue());
/*     */     }
/*     */     
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 211 */       return createSchemaNode("number", true);
/*     */     }
/*     */   }
/*     */   
/*     */   @JacksonStdImpl
/*     */   public static final class FloatSerializer
/*     */     extends ScalarSerializerBase<Float>
/*     */   {
/* 219 */     static final FloatSerializer instance = new FloatSerializer();
/*     */     
/* 221 */     public FloatSerializer() { super(); }
/*     */     
/*     */ 
/*     */     public void serialize(Float value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 227 */       jgen.writeNumber(value.floatValue());
/*     */     }
/*     */     
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 233 */       return createSchemaNode("number", true);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class DoubleSerializer
/*     */     extends StdSerializers.NonTypedScalarSerializer<Double>
/*     */   {
/* 248 */     static final DoubleSerializer instance = new DoubleSerializer();
/*     */     
/* 250 */     public DoubleSerializer() { super(); }
/*     */     
/*     */ 
/*     */     public void serialize(Double value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 256 */       jgen.writeNumber(value.doubleValue());
/*     */     }
/*     */     
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 262 */       return createSchemaNode("number", true);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class NumberSerializer
/*     */     extends ScalarSerializerBase<Number>
/*     */   {
/* 274 */     public static final NumberSerializer instance = new NumberSerializer();
/*     */     
/* 276 */     public NumberSerializer() { super(); }
/*     */     
/*     */ 
/*     */ 
/*     */     public void serialize(Number value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 283 */       if ((value instanceof BigDecimal)) {
/* 284 */         jgen.writeNumber((BigDecimal)value);
/* 285 */       } else if ((value instanceof BigInteger)) {
/* 286 */         jgen.writeNumber((BigInteger)value);
/*     */ 
/*     */ 
/*     */ 
/*     */       }
/* 291 */       else if ((value instanceof Integer)) {
/* 292 */         jgen.writeNumber(value.intValue());
/* 293 */       } else if ((value instanceof Long)) {
/* 294 */         jgen.writeNumber(value.longValue());
/* 295 */       } else if ((value instanceof Double)) {
/* 296 */         jgen.writeNumber(value.doubleValue());
/* 297 */       } else if ((value instanceof Float)) {
/* 298 */         jgen.writeNumber(value.floatValue());
/* 299 */       } else if (((value instanceof Byte)) || ((value instanceof Short))) {
/* 300 */         jgen.writeNumber(value.intValue());
/*     */       }
/*     */       else {
/* 303 */         jgen.writeNumber(value.toString());
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 310 */       return createSchemaNode("number", true);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class CalendarSerializer
/*     */     extends ScalarSerializerBase<Calendar>
/*     */   {
/* 329 */     public static final CalendarSerializer instance = new CalendarSerializer();
/*     */     
/* 331 */     public CalendarSerializer() { super(); }
/*     */     
/*     */ 
/*     */     public void serialize(Calendar value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 337 */       provider.defaultSerializeDateValue(value.getTimeInMillis(), jgen);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 344 */       return createSchemaNode(provider.isEnabled(SerializationConfig.Feature.WRITE_DATES_AS_TIMESTAMPS) ? "number" : "string", true);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class UtilDateSerializer
/*     */     extends ScalarSerializerBase<java.util.Date>
/*     */   {
/* 357 */     public static final UtilDateSerializer instance = new UtilDateSerializer();
/*     */     
/* 359 */     public UtilDateSerializer() { super(); }
/*     */     
/*     */ 
/*     */     public void serialize(java.util.Date value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 365 */       provider.defaultSerializeDateValue(value, jgen);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 372 */       return createSchemaNode(provider.isEnabled(SerializationConfig.Feature.WRITE_DATES_AS_TIMESTAMPS) ? "number" : "string", true);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class SqlDateSerializer
/*     */     extends ScalarSerializerBase<java.sql.Date>
/*     */   {
/*     */     public SqlDateSerializer()
/*     */     {
/* 386 */       super();
/*     */     }
/*     */     
/*     */     public void serialize(java.sql.Date value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 392 */       jgen.writeString(value.toString());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 399 */       return createSchemaNode("string", true);
/*     */     }
/*     */   }
/*     */   
/*     */   @JacksonStdImpl
/*     */   public static final class SqlTimeSerializer extends ScalarSerializerBase<Time>
/*     */   {
/*     */     public SqlTimeSerializer() {
/* 407 */       super();
/*     */     }
/*     */     
/*     */     public void serialize(Time value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 413 */       jgen.writeString(value.toString());
/*     */     }
/*     */     
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 419 */       return createSchemaNode("string", true);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class SerializableSerializer
/*     */     extends SerializerBase<JsonSerializable>
/*     */   {
/* 441 */     protected static final SerializableSerializer instance = new SerializableSerializer();
/*     */     
/* 443 */     private SerializableSerializer() { super(); }
/*     */     
/*     */ 
/*     */     public void serialize(JsonSerializable value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 449 */       value.serialize(jgen, provider);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public final void serializeWithType(JsonSerializable value, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 461 */       if ((value instanceof JsonSerializableWithType)) {
/* 462 */         ((JsonSerializableWithType)value).serializeWithType(jgen, provider, typeSer);
/*     */       } else {
/* 464 */         serialize(value, jgen, provider);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */       throws JsonMappingException
/*     */     {
/* 472 */       ObjectNode objectNode = createObjectNode();
/* 473 */       String schemaType = "any";
/* 474 */       String objectProperties = null;
/* 475 */       String itemDefinition = null;
/* 476 */       if (typeHint != null) {
/* 477 */         Class<?> rawClass = TypeFactory.type(typeHint).getRawClass();
/* 478 */         if (rawClass.isAnnotationPresent(JsonSerializableSchema.class)) {
/* 479 */           JsonSerializableSchema schemaInfo = (JsonSerializableSchema)rawClass.getAnnotation(JsonSerializableSchema.class);
/* 480 */           schemaType = schemaInfo.schemaType();
/* 481 */           if (!"##irrelevant".equals(schemaInfo.schemaObjectPropertiesDefinition())) {
/* 482 */             objectProperties = schemaInfo.schemaObjectPropertiesDefinition();
/*     */           }
/* 484 */           if (!"##irrelevant".equals(schemaInfo.schemaItemDefinition())) {
/* 485 */             itemDefinition = schemaInfo.schemaItemDefinition();
/*     */           }
/*     */         }
/*     */       }
/* 489 */       objectNode.put("type", schemaType);
/* 490 */       if (objectProperties != null) {
/*     */         try {
/* 492 */           objectNode.put("properties", (JsonNode)new ObjectMapper().readValue(objectProperties, JsonNode.class));
/*     */         } catch (IOException e) {
/* 494 */           throw new IllegalStateException(e);
/*     */         }
/*     */       }
/* 497 */       if (itemDefinition != null) {
/*     */         try {
/* 499 */           objectNode.put("items", (JsonNode)new ObjectMapper().readValue(itemDefinition, JsonNode.class));
/*     */         } catch (IOException e) {
/* 501 */           throw new IllegalStateException(e);
/*     */         }
/*     */       }
/*     */       
/*     */ 
/* 506 */       return objectNode;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class SerializableWithTypeSerializer
/*     */     extends SerializerBase<JsonSerializableWithType>
/*     */   {
/* 520 */     protected static final SerializableWithTypeSerializer instance = new SerializableWithTypeSerializer();
/*     */     
/* 522 */     private SerializableWithTypeSerializer() { super(); }
/*     */     
/*     */ 
/*     */ 
/*     */     public void serialize(JsonSerializableWithType value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 529 */       value.serialize(jgen, provider);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     public final void serializeWithType(JsonSerializableWithType value, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 537 */       value.serializeWithType(jgen, provider, typeSer);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */       throws JsonMappingException
/*     */     {
/* 545 */       ObjectNode objectNode = createObjectNode();
/* 546 */       String schemaType = "any";
/* 547 */       String objectProperties = null;
/* 548 */       String itemDefinition = null;
/* 549 */       if (typeHint != null) {
/* 550 */         Class<?> rawClass = TypeFactory.rawClass(typeHint);
/* 551 */         if (rawClass.isAnnotationPresent(JsonSerializableSchema.class)) {
/* 552 */           JsonSerializableSchema schemaInfo = (JsonSerializableSchema)rawClass.getAnnotation(JsonSerializableSchema.class);
/* 553 */           schemaType = schemaInfo.schemaType();
/* 554 */           if (!"##irrelevant".equals(schemaInfo.schemaObjectPropertiesDefinition())) {
/* 555 */             objectProperties = schemaInfo.schemaObjectPropertiesDefinition();
/*     */           }
/* 557 */           if (!"##irrelevant".equals(schemaInfo.schemaItemDefinition())) {
/* 558 */             itemDefinition = schemaInfo.schemaItemDefinition();
/*     */           }
/*     */         }
/*     */       }
/* 562 */       objectNode.put("type", schemaType);
/* 563 */       if (objectProperties != null) {
/*     */         try {
/* 565 */           objectNode.put("properties", (JsonNode)new ObjectMapper().readValue(objectProperties, JsonNode.class));
/*     */         } catch (IOException e) {
/* 567 */           throw new IllegalStateException(e);
/*     */         }
/*     */       }
/* 570 */       if (itemDefinition != null) {
/*     */         try {
/* 572 */           objectNode.put("items", (JsonNode)new ObjectMapper().readValue(itemDefinition, JsonNode.class));
/*     */         } catch (IOException e) {
/* 574 */           throw new IllegalStateException(e);
/*     */         }
/*     */       }
/*     */       
/*     */ 
/* 579 */       return objectNode;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @JacksonStdImpl
/*     */   public static final class TokenBufferSerializer
/*     */     extends SerializerBase<TokenBuffer>
/*     */   {
/*     */     public TokenBufferSerializer()
/*     */     {
/* 595 */       super();
/*     */     }
/*     */     
/*     */     public void serialize(TokenBuffer value, JsonGenerator jgen, SerializerProvider provider)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 601 */       value.serialize(jgen);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public final void serializeWithType(TokenBuffer value, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer)
/*     */       throws IOException, JsonGenerationException
/*     */     {
/* 621 */       typeSer.writeTypePrefixForScalar(value, jgen);
/* 622 */       serialize(value, jgen, provider);
/* 623 */       typeSer.writeTypeSuffixForScalar(value, jgen);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     {
/* 633 */       return createSchemaNode("any", true);
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ser\StdSerializers.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */