package prefuse.data.column;

import java.util.Arrays;
import prefuse.data.DataReadOnlyException;
import prefuse.data.DataTypeException;

public class DoubleColumn
  extends AbstractColumn
{
  private double[] m_values;
  private int m_size;
  
  public DoubleColumn()
  {
    this(0, 10, 0.0D);
  }
  
  public DoubleColumn(int paramInt)
  {
    this(paramInt, paramInt, 0.0D);
  }
  
  public DoubleColumn(int paramInt1, int paramInt2, double paramDouble)
  {
    super(Double.TYPE, new Double(paramDouble));
    if (paramInt2 < paramInt1) {
      throw new IllegalArgumentException("Capacity value can not be less than the row count.");
    }
    this.m_values = new double[paramInt2];
    Arrays.fill(this.m_values, paramDouble);
    this.m_size = paramInt1;
  }
  
  public int getRowCount()
  {
    return this.m_size;
  }
  
  public void setMaximumRow(int paramInt)
  {
    if (paramInt > this.m_values.length)
    {
      int i = Math.max(3 * this.m_values.length / 2 + 1, paramInt);
      double[] arrayOfDouble = new double[i];
      System.arraycopy(this.m_values, 0, arrayOfDouble, 0, this.m_size);
      Arrays.fill(arrayOfDouble, this.m_size, i, ((Double)this.m_defaultValue).doubleValue());
      this.m_values = arrayOfDouble;
    }
    this.m_size = paramInt;
  }
  
  public Object get(int paramInt)
  {
    return new Double(getDouble(paramInt));
  }
  
  public void set(Object paramObject, int paramInt)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if (paramObject != null)
    {
      if ((paramObject instanceof Number)) {
        setDouble(((Number)paramObject).doubleValue(), paramInt);
      } else if ((paramObject instanceof String)) {
        setString((String)paramObject, paramInt);
      } else {
        throw new DataTypeException(paramObject.getClass());
      }
    }
    else {
      throw new DataTypeException("Column does not accept null values");
    }
  }
  
  public double getDouble(int paramInt)
    throws DataTypeException
  {
    if ((paramInt < 0) || (paramInt > this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    return this.m_values[paramInt];
  }
  
  public void setDouble(double paramDouble, int paramInt)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if ((paramInt < 0) || (paramInt >= this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    double d = this.m_values[paramInt];
    if (d == paramDouble) {
      return;
    }
    this.m_values[paramInt] = paramDouble;
    fireColumnEvent(paramInt, d);
  }
  
  public String getString(int paramInt)
    throws DataTypeException
  {
    return String.valueOf(getDouble(paramInt));
  }
  
  public void setString(String paramString, int paramInt)
    throws DataTypeException
  {
    setDouble(Double.parseDouble(paramString), paramInt);
  }
  
  public int getInt(int paramInt)
    throws DataTypeException
  {
    return (int)getDouble(paramInt);
  }
  
  public void setInt(int paramInt1, int paramInt2)
    throws DataTypeException
  {
    setDouble(paramInt1, paramInt2);
  }
  
  public long getLong(int paramInt)
    throws DataTypeException
  {
    return getDouble(paramInt);
  }
  
  public void setLong(long paramLong, int paramInt)
    throws DataTypeException
  {
    setDouble(paramLong, paramInt);
  }
  
  public float getFloat(int paramInt)
    throws DataTypeException
  {
    return (float)getDouble(paramInt);
  }
  
  public void setFloat(float paramFloat, int paramInt)
    throws DataTypeException
  {
    setDouble(paramFloat, paramInt);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\column\DoubleColumn.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */