package prefuse.util.collections;

import java.io.PrintStream;
import java.util.Comparator;
import java.util.ConcurrentModificationException;
import java.util.NoSuchElementException;

public abstract class AbstractTreeMap
  implements IntSortedMap
{
  protected static final boolean RED = false;
  protected static final boolean BLACK = true;
  protected static final Entry NIL = new Entry(Integer.MIN_VALUE);
  protected LiteralComparator cmp = null;
  protected Entry root = NIL;
  protected boolean allowDuplicates;
  protected int size = 0;
  protected int unique = 0;
  protected int modCount = 0;
  protected int lastOrder = 0;
  
  public AbstractTreeMap(LiteralComparator paramLiteralComparator, boolean paramBoolean)
  {
    this.cmp = (paramLiteralComparator == null ? DefaultLiteralComparator.getInstance() : paramLiteralComparator);
    this.allowDuplicates = paramBoolean;
  }
  
  public boolean isAllowDuplicates()
  {
    return this.allowDuplicates;
  }
  
  public int size()
  {
    return this.size;
  }
  
  public boolean isEmpty()
  {
    return this.root == NIL;
  }
  
  public Comparator comparator()
  {
    return this.cmp;
  }
  
  public void clear()
  {
    this.modCount += 1;
    this.size = 0;
    this.root = NIL;
  }
  
  public int getMinimum()
  {
    return minimum(this.root).getValue();
  }
  
  public int getMaximum()
  {
    return maximum(this.root).getValue();
  }
  
  public int getMedian()
  {
    Entry localEntry = minimum(this.root);
    int i = 0;
    while (i < this.size / 2)
    {
      i++;
      localEntry = successor(localEntry);
    }
    return localEntry.getValue();
  }
  
  public int getUniqueCount()
  {
    return this.unique;
  }
  
  public boolean containsValue(int paramInt)
  {
    return this.root == NIL ? false : containsValue(this.root, paramInt);
  }
  
  private boolean containsValue(Entry paramEntry, int paramInt)
  {
    if (paramEntry.val == paramInt) {
      return true;
    }
    return ((paramEntry.left != NIL) && (containsValue(paramEntry.left, paramInt))) || ((paramEntry.right != NIL) && (containsValue(paramEntry.right, paramInt)));
  }
  
  public IntIterator valueIterator(boolean paramBoolean)
  {
    return new ValueIterator(new EntryIterator(!paramBoolean));
  }
  
  protected void incrementSize(boolean paramBoolean)
  {
    this.size += 1;
    this.modCount += 1;
    if (paramBoolean) {
      this.unique += 1;
    }
  }
  
  protected void decrementSize(boolean paramBoolean)
  {
    this.size -= 1;
    this.modCount += 1;
    if (paramBoolean) {
      this.unique -= 1;
    }
  }
  
  protected abstract int compare(Entry paramEntry1, Entry paramEntry2);
  
  protected Entry find(Entry paramEntry)
  {
    Entry localEntry = this.root;
    while (localEntry != NIL)
    {
      int i = compare(paramEntry, localEntry);
      if (i == 0) {
        return localEntry;
      }
      if (i < 0) {
        localEntry = localEntry.left;
      } else {
        localEntry = localEntry.right;
      }
    }
    return localEntry;
  }
  
  protected Entry findPredecessor(Entry paramEntry)
  {
    Entry localEntry1 = this.root;
    while (localEntry1 != NIL)
    {
      int i = compare(paramEntry, localEntry1);
      if (i > 0)
      {
        if (localEntry1.right == NIL) {
          return localEntry1;
        }
        localEntry1 = localEntry1.right;
      }
      else if (localEntry1.left != NIL)
      {
        localEntry1 = localEntry1.left;
      }
      else
      {
        Entry localEntry2 = localEntry1.p;
        Entry localEntry3 = localEntry1;
        while ((localEntry2 != NIL) && (localEntry3 == localEntry2.left))
        {
          localEntry3 = localEntry2;
          localEntry2 = localEntry2.p;
        }
        return localEntry2;
      }
    }
    return localEntry1;
  }
  
  protected Entry findCeiling(Entry paramEntry)
  {
    Entry localEntry1 = this.root;
    while (localEntry1 != NIL)
    {
      int i = compare(paramEntry, localEntry1);
      if (i == 0) {
        return localEntry1;
      }
      if (i < 0)
      {
        if (localEntry1.left != NIL) {
          localEntry1 = localEntry1.left;
        } else {
          return localEntry1;
        }
      }
      else if (localEntry1.right != NIL)
      {
        localEntry1 = localEntry1.right;
      }
      else
      {
        Entry localEntry2 = localEntry1.p;
        Entry localEntry3 = localEntry1;
        while ((localEntry2 != NIL) && (localEntry3 == localEntry2.right))
        {
          localEntry3 = localEntry2;
          localEntry2 = localEntry2.p;
        }
        return localEntry2;
      }
    }
    return localEntry1;
  }
  
  protected Entry minimum(Entry paramEntry)
  {
    while (paramEntry.left != NIL) {
      paramEntry = paramEntry.left;
    }
    return paramEntry;
  }
  
  protected Entry maximum(Entry paramEntry)
  {
    while (paramEntry.right != NIL) {
      paramEntry = paramEntry.right;
    }
    return paramEntry;
  }
  
  protected Entry successor(Entry paramEntry)
  {
    if (paramEntry.right != NIL) {
      return minimum(paramEntry.right);
    }
    for (Entry localEntry = paramEntry.p; (localEntry != NIL) && (paramEntry == localEntry.right); localEntry = localEntry.p) {
      paramEntry = localEntry;
    }
    return localEntry;
  }
  
  protected Entry predecessor(Entry paramEntry)
  {
    if (paramEntry.left != NIL) {
      return maximum(paramEntry.left);
    }
    for (Entry localEntry = paramEntry.p; (localEntry != NIL) && (paramEntry == localEntry.left); localEntry = localEntry.p) {
      paramEntry = localEntry;
    }
    return localEntry;
  }
  
  protected void rotateLeft(Entry paramEntry)
  {
    Entry localEntry = paramEntry.right;
    paramEntry.right = localEntry.left;
    if (localEntry.left != NIL) {
      localEntry.left.p = paramEntry;
    }
    localEntry.p = paramEntry.p;
    if (paramEntry.p == NIL) {
      this.root = localEntry;
    } else if (paramEntry.p.left == paramEntry) {
      paramEntry.p.left = localEntry;
    } else {
      paramEntry.p.right = localEntry;
    }
    localEntry.left = paramEntry;
    paramEntry.p = localEntry;
  }
  
  protected void rotateRight(Entry paramEntry)
  {
    Entry localEntry = paramEntry.left;
    paramEntry.left = localEntry.right;
    if (localEntry.right != NIL) {
      localEntry.right.p = paramEntry;
    }
    localEntry.p = paramEntry.p;
    if (paramEntry.p == NIL) {
      this.root = localEntry;
    } else if (paramEntry.p.right == paramEntry) {
      paramEntry.p.right = localEntry;
    } else {
      paramEntry.p.left = localEntry;
    }
    localEntry.right = paramEntry;
    paramEntry.p = localEntry;
  }
  
  protected void fixUpInsert(Entry paramEntry)
  {
    paramEntry.color = false;
    while ((paramEntry != NIL) && (paramEntry != this.root) && (!paramEntry.p.color))
    {
      Entry localEntry;
      if (paramEntry.p == paramEntry.p.p.left)
      {
        localEntry = paramEntry.p.p.right;
        if (!localEntry.color)
        {
          paramEntry.p.color = true;
          localEntry.color = true;
          paramEntry.p.p.color = false;
          paramEntry = paramEntry.p.p;
        }
        else
        {
          if (paramEntry == paramEntry.p.right)
          {
            paramEntry = paramEntry.p;
            rotateLeft(paramEntry);
          }
          paramEntry.p.color = true;
          paramEntry.p.p.color = false;
          if (paramEntry.p.p != NIL) {
            rotateRight(paramEntry.p.p);
          }
        }
      }
      else
      {
        localEntry = paramEntry.p.p.left;
        if (!localEntry.color)
        {
          paramEntry.p.color = true;
          localEntry.color = true;
          paramEntry.p.p.color = false;
          paramEntry = paramEntry.p.p;
        }
        else
        {
          if (paramEntry == paramEntry.p.left)
          {
            paramEntry = paramEntry.p;
            rotateRight(paramEntry);
          }
          paramEntry.p.color = true;
          paramEntry.p.p.color = false;
          if (paramEntry.p.p != NIL) {
            rotateLeft(paramEntry.p.p);
          }
        }
      }
    }
    this.root.color = true;
  }
  
  protected void fixUpRemove(Entry paramEntry)
  {
    while ((paramEntry != this.root) && (paramEntry.color == true))
    {
      Entry localEntry;
      if (paramEntry == paramEntry.p.left)
      {
        localEntry = paramEntry.p.right;
        if (!localEntry.color)
        {
          localEntry.color = true;
          paramEntry.p.color = false;
          rotateLeft(paramEntry.p);
          localEntry = paramEntry.p.right;
        }
        if ((localEntry.left.color == true) && (localEntry.right.color == true))
        {
          localEntry.color = false;
          paramEntry = paramEntry.p;
        }
        else
        {
          if (localEntry.right.color == true)
          {
            localEntry.left.color = true;
            localEntry.color = false;
            rotateRight(localEntry);
            localEntry = paramEntry.p.right;
          }
          localEntry.color = paramEntry.p.color;
          paramEntry.p.color = true;
          localEntry.right.color = true;
          rotateLeft(paramEntry.p);
          paramEntry = this.root;
        }
      }
      else
      {
        localEntry = paramEntry.p.left;
        if (!localEntry.color)
        {
          localEntry.color = true;
          paramEntry.p.color = false;
          rotateRight(paramEntry.p);
          localEntry = paramEntry.p.left;
        }
        if ((localEntry.right.color == true) && (localEntry.left.color == true))
        {
          localEntry.color = false;
          paramEntry = paramEntry.p;
        }
        else
        {
          if (localEntry.left.color == true)
          {
            localEntry.right.color = true;
            localEntry.color = false;
            rotateLeft(localEntry);
            localEntry = paramEntry.p.left;
          }
          localEntry.color = paramEntry.p.color;
          paramEntry.p.color = true;
          localEntry.left.color = true;
          rotateRight(paramEntry.p);
          paramEntry = this.root;
        }
      }
    }
    paramEntry.color = true;
  }
  
  protected void remove(Entry paramEntry)
  {
    boolean bool = (!paramEntry.keyEquals(paramEntry.left)) && (!paramEntry.keyEquals(paramEntry.right)) && (!paramEntry.keyEquals(paramEntry.p));
    Entry localEntry1 = (paramEntry.left != NIL) && (paramEntry.right != NIL) ? successor(paramEntry) : paramEntry;
    Entry localEntry2 = localEntry1.left != NIL ? localEntry1.left : localEntry1.right;
    localEntry2.p = localEntry1.p;
    if (localEntry1.p == NIL) {
      this.root = localEntry2;
    } else if (localEntry1 == localEntry1.p.left) {
      localEntry1.p.left = localEntry2;
    } else {
      localEntry1.p.right = localEntry2;
    }
    if (localEntry1 != paramEntry) {
      paramEntry.copyFields(localEntry1);
    }
    if (localEntry1.color == true) {
      fixUpRemove(localEntry2);
    }
    decrementSize(bool);
  }
  
  static
  {
    NIL.left = (NIL.right = NIL.p = NIL);
  }
  
  protected class ValueIterator
    extends IntIterator
  {
    AbstractTreeMap.EntryIterator m_iter;
    
    public ValueIterator(AbstractTreeMap.EntryIterator paramEntryIterator)
    {
      this.m_iter = paramEntryIterator;
    }
    
    public boolean hasNext()
    {
      return this.m_iter.hasNext();
    }
    
    public int nextInt()
    {
      return this.m_iter.nextEntry().val;
    }
    
    public void remove()
    {
      this.m_iter.remove();
    }
  }
  
  protected class KeyIterator
    extends AbstractTreeMap.EntryIterator
  {
    public KeyIterator()
    {
      super(false);
    }
    
    public KeyIterator(AbstractTreeMap.Entry paramEntry1, AbstractTreeMap.Entry paramEntry2)
    {
      super(paramEntry1, paramEntry2);
    }
    
    public Object next()
    {
      return nextEntry().getKey();
    }
  }
  
  protected class EntryIterator
    extends AbstractLiteralIterator
  {
    private int expectedModCount = AbstractTreeMap.this.modCount;
    private AbstractTreeMap.Entry lastReturned = AbstractTreeMap.NIL;
    private boolean reverse = false;
    AbstractTreeMap.Entry next;
    AbstractTreeMap.Entry end;
    
    EntryIterator(boolean paramBoolean)
    {
      this.next = (paramBoolean ? AbstractTreeMap.this.maximum(AbstractTreeMap.this.root) : AbstractTreeMap.this.minimum(AbstractTreeMap.this.root));
      this.end = AbstractTreeMap.NIL;
    }
    
    EntryIterator(AbstractTreeMap.Entry paramEntry1, AbstractTreeMap.Entry paramEntry2)
    {
      this.next = paramEntry1;
      this.end = paramEntry2;
      this.reverse = (paramEntry1 == AbstractTreeMap.NIL);
    }
    
    public boolean hasNext()
    {
      return this.next != this.end;
    }
    
    final AbstractTreeMap.Entry nextEntry()
    {
      if (!hasNext()) {
        throw new NoSuchElementException();
      }
      if (AbstractTreeMap.this.modCount != this.expectedModCount) {
        throw new ConcurrentModificationException();
      }
      this.lastReturned = this.next;
      this.next = (this.reverse ? AbstractTreeMap.this.predecessor(this.next) : AbstractTreeMap.this.successor(this.next));
      if (this.lastReturned == AbstractTreeMap.NIL) {
        System.err.println("Encountered NIL in iteration!");
      }
      return this.lastReturned;
    }
    
    public Object next()
    {
      return nextEntry();
    }
    
    public void remove()
    {
      if (this.lastReturned == AbstractTreeMap.NIL) {
        throw new IllegalStateException();
      }
      if (AbstractTreeMap.this.modCount != this.expectedModCount) {
        throw new ConcurrentModificationException();
      }
      if ((this.lastReturned.left != AbstractTreeMap.NIL) && (this.lastReturned.right != AbstractTreeMap.NIL)) {
        this.next = this.lastReturned;
      }
      AbstractTreeMap.this.remove(this.lastReturned);
      this.expectedModCount += 1;
      this.lastReturned = AbstractTreeMap.NIL;
    }
  }
  
  public static class Entry
  {
    int val;
    int order;
    Entry left = null;
    Entry right = null;
    Entry p;
    boolean color = true;
    
    public Entry(int paramInt)
    {
      this.val = paramInt;
    }
    
    public Entry(int paramInt1, Entry paramEntry, int paramInt2)
    {
      this.val = paramInt1;
      this.p = paramEntry;
      this.order = paramInt2;
      this.left = AbstractTreeMap.NIL;
      this.right = AbstractTreeMap.NIL;
    }
    
    public int getIntKey()
    {
      throw new UnsupportedOperationException("Unsupported");
    }
    
    public long getLongKey()
    {
      throw new UnsupportedOperationException("Unsupported");
    }
    
    public float getFloatKey()
    {
      throw new UnsupportedOperationException("Unsupported");
    }
    
    public double getDoubleKey()
    {
      throw new UnsupportedOperationException("Unsupported");
    }
    
    public Object getKey()
    {
      return null;
    }
    
    public int getValue()
    {
      return this.val;
    }
    
    public int getOrder()
    {
      return this.order;
    }
    
    public int setValue(int paramInt)
    {
      int i = this.val;
      this.val = paramInt;
      return i;
    }
    
    public boolean keyEquals(Entry paramEntry)
    {
      Object localObject = getKey();
      return localObject == null ? false : localObject == paramEntry.getKey() ? true : localObject.equals(paramEntry.getKey());
    }
    
    public boolean equals(Object paramObject)
    {
      if (!(paramObject instanceof Entry)) {
        return false;
      }
      Entry localEntry = (Entry)paramObject;
      return (this.val == localEntry.val) && (getKey() == localEntry.getKey());
    }
    
    public int hashCode()
    {
      int i = getKey().hashCode();
      int j = this.val;
      return i ^ j;
    }
    
    public String toString()
    {
      return getKey() + "=" + this.val;
    }
    
    public void copyFields(Entry paramEntry)
    {
      this.val = paramEntry.val;
      this.order = paramEntry.order;
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\collections\AbstractTreeMap.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */