package prefuse.util.collections;

public class FloatIntTreeMap
  extends AbstractTreeMap
  implements FloatIntSortedMap
{
  private FloatEntry dummy = new FloatEntry(Float.MIN_VALUE, Integer.MAX_VALUE, NIL, 0);
  
  public FloatIntTreeMap()
  {
    this(null, false);
  }
  
  public FloatIntTreeMap(boolean paramBoolean)
  {
    this(null, paramBoolean);
  }
  
  public FloatIntTreeMap(LiteralComparator paramLiteralComparator)
  {
    this(paramLiteralComparator, false);
  }
  
  public FloatIntTreeMap(LiteralComparator paramLiteralComparator, boolean paramBoolean)
  {
    super(paramLiteralComparator, paramBoolean);
  }
  
  public void clear()
  {
    this.modCount += 1;
    this.size = 0;
    this.root = NIL;
  }
  
  public boolean containsKey(float paramFloat)
  {
    return find(paramFloat, 0) != NIL;
  }
  
  public int get(float paramFloat)
  {
    AbstractTreeMap.Entry localEntry = find(paramFloat, 0);
    return localEntry == NIL ? Integer.MIN_VALUE : localEntry.val;
  }
  
  public int put(float paramFloat, int paramInt)
  {
    AbstractTreeMap.Entry localEntry = this.root;
    this.lastOrder = 0;
    if (localEntry == NIL)
    {
      incrementSize(true);
      this.root = new FloatEntry(paramFloat, paramInt, NIL, this.lastOrder);
      return Integer.MIN_VALUE;
    }
    this.dummy.key = paramFloat;
    this.dummy.order = Integer.MAX_VALUE;
    for (;;)
    {
      int i = compare(this.dummy, localEntry);
      if (i == 0) {
        return localEntry.setValue(paramInt);
      }
      if (i < 0)
      {
        if (localEntry.left != NIL)
        {
          localEntry = localEntry.left;
        }
        else
        {
          incrementSize(this.lastOrder == 0);
          localEntry.left = new FloatEntry(paramFloat, paramInt, localEntry, this.lastOrder);
          fixUpInsert(localEntry.left);
          return Integer.MIN_VALUE;
        }
      }
      else if (localEntry.right != NIL)
      {
        localEntry = localEntry.right;
      }
      else
      {
        incrementSize(this.lastOrder == 0);
        localEntry.right = new FloatEntry(paramFloat, paramInt, localEntry, this.lastOrder);
        fixUpInsert(localEntry.right);
        return Integer.MIN_VALUE;
      }
    }
  }
  
  public int remove(float paramFloat)
  {
    AbstractTreeMap.Entry localEntry;
    if (this.allowDuplicates) {
      localEntry = findPredecessor(paramFloat, Integer.MAX_VALUE);
    } else {
      localEntry = find(paramFloat, 0);
    }
    if (localEntry == NIL) {
      return Integer.MIN_VALUE;
    }
    int i = localEntry.val;
    remove(localEntry);
    return i;
  }
  
  public int remove(float paramFloat, int paramInt)
  {
    AbstractTreeMap.Entry localEntry = findCeiling(paramFloat, 0);
    if ((localEntry != NIL) && (localEntry.getFloatKey() != paramFloat)) {
      localEntry = successor(localEntry);
    }
    if ((localEntry == NIL) || (localEntry.getFloatKey() != paramFloat)) {
      return Integer.MIN_VALUE;
    }
    while ((localEntry.val != paramInt) && (localEntry != NIL)) {
      localEntry = successor(localEntry);
    }
    if (localEntry == NIL) {
      return Integer.MIN_VALUE;
    }
    remove(localEntry);
    return paramInt;
  }
  
  public float firstKey()
  {
    return minimum(this.root).getFloatKey();
  }
  
  public float lastKey()
  {
    return maximum(this.root).getFloatKey();
  }
  
  public LiteralIterator keyIterator()
  {
    return new KeyIterator();
  }
  
  public LiteralIterator keyRangeIterator(float paramFloat1, boolean paramBoolean1, float paramFloat2, boolean paramBoolean2)
  {
    AbstractTreeMap.Entry localEntry1;
    AbstractTreeMap.Entry localEntry2;
    if (this.cmp.compare(paramFloat1, paramFloat2) <= 0)
    {
      localEntry1 = findCeiling(paramFloat1, paramBoolean1 ? 0 : Integer.MAX_VALUE);
      localEntry2 = findCeiling(paramFloat2, paramBoolean2 ? Integer.MAX_VALUE : 0);
    }
    else
    {
      localEntry1 = findCeiling(paramFloat1, paramBoolean1 ? Integer.MAX_VALUE : 0);
      localEntry1 = predecessor(localEntry1);
      localEntry2 = findCeiling(paramFloat2, paramBoolean2 ? 0 : Integer.MAX_VALUE);
      localEntry2 = predecessor(localEntry2);
    }
    return new KeyIterator(localEntry1, localEntry2);
  }
  
  public IntIterator valueRangeIterator(float paramFloat1, boolean paramBoolean1, float paramFloat2, boolean paramBoolean2)
  {
    return new AbstractTreeMap.ValueIterator(this, (AbstractTreeMap.EntryIterator)keyRangeIterator(paramFloat1, paramBoolean1, paramFloat2, paramBoolean2));
  }
  
  protected int compare(AbstractTreeMap.Entry paramEntry1, AbstractTreeMap.Entry paramEntry2)
  {
    int i = this.cmp.compare(paramEntry1.getFloatKey(), paramEntry2.getFloatKey());
    if ((this.allowDuplicates) && (i == 0))
    {
      i = paramEntry1.order > paramEntry2.order ? 1 : paramEntry1.order < paramEntry2.order ? -1 : 0;
      this.lastOrder = (1 + (i < 0 ? paramEntry1.order : paramEntry2.order));
    }
    return i;
  }
  
  private AbstractTreeMap.Entry find(float paramFloat, int paramInt)
  {
    this.dummy.key = paramFloat;
    this.dummy.order = paramInt;
    AbstractTreeMap.Entry localEntry = find(this.dummy);
    return localEntry;
  }
  
  private AbstractTreeMap.Entry findPredecessor(float paramFloat, int paramInt)
  {
    this.dummy.key = paramFloat;
    this.dummy.order = paramInt;
    AbstractTreeMap.Entry localEntry = findPredecessor(this.dummy);
    return localEntry;
  }
  
  private AbstractTreeMap.Entry findCeiling(float paramFloat, int paramInt)
  {
    this.dummy.key = paramFloat;
    this.dummy.order = paramInt;
    AbstractTreeMap.Entry localEntry = findCeiling(this.dummy);
    return localEntry;
  }
  
  private class KeyIterator
    extends AbstractTreeMap.KeyIterator
  {
    public KeyIterator()
    {
      super();
    }
    
    public KeyIterator(AbstractTreeMap.Entry paramEntry1, AbstractTreeMap.Entry paramEntry2)
    {
      super(paramEntry1, paramEntry2);
    }
    
    public boolean isFloatSupported()
    {
      return true;
    }
    
    public float nextFloat()
    {
      return nextEntry().getFloatKey();
    }
  }
  
  static class FloatEntry
    extends AbstractTreeMap.Entry
  {
    float key;
    
    public FloatEntry(float paramFloat, int paramInt)
    {
      super();
      this.key = paramFloat;
    }
    
    public FloatEntry(float paramFloat, int paramInt1, AbstractTreeMap.Entry paramEntry, int paramInt2)
    {
      super(paramEntry, paramInt2);
      this.key = paramFloat;
    }
    
    public float getFloatKey()
    {
      return this.key;
    }
    
    public Object getKey()
    {
      return new Float(this.key);
    }
    
    public boolean keyEquals(AbstractTreeMap.Entry paramEntry)
    {
      return ((paramEntry instanceof FloatEntry)) && (this.key == ((FloatEntry)paramEntry).key);
    }
    
    public boolean equals(Object paramObject)
    {
      if (!(paramObject instanceof FloatEntry)) {
        return false;
      }
      FloatEntry localFloatEntry = (FloatEntry)paramObject;
      return (this.key == localFloatEntry.key) && (this.val == localFloatEntry.val);
    }
    
    public int hashCode()
    {
      int i = Float.floatToIntBits(this.key);
      int j = this.val;
      return i ^ j ^ this.order;
    }
    
    public String toString()
    {
      return this.key + "=" + this.val;
    }
    
    public void copyFields(AbstractTreeMap.Entry paramEntry)
    {
      super.copyFields(paramEntry);
      this.key = paramEntry.getFloatKey();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\collections\FloatIntTreeMap.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */